/*****************************************************************************/
/* TestFile.h                             Copyright (c) Ladislav Zezula 2003 */
/*---------------------------------------------------------------------------*/
/* Definitions for file access testing application                           */
/*---------------------------------------------------------------------------*/
/*   Date    Ver   Who  Comment                                              */
/* --------  ----  ---  -------                                              */
/* 14.07.03  1.00  Lad  The first version of TestFile.h                      */
/*****************************************************************************/

#ifndef __TESTFILE_H__
#define __TESTFILE_H__


#define UNICODE
#define _UNICODE
#include <tchar.h>
#include <stdio.h>
#include "ntstatus.h"

#define WIN32_NO_STATUS 
#include <windows.h>
#include <windowsx.h>
#include <ShlObj.h>

#include "ntdefs.h"
#include "Utils.h"

//-----------------------------------------------------------------------------
// Defines

#define IsHandleValid(h)   (h != NULL && h != INVALID_HANDLE_VALUE)
#define IsHandleInvalid(h) (h == NULL || h == INVALID_HANDLE_VALUE)


#define MAX_NT_PATH 32767               // Maximum path name length in NT is 32767

//-----------------------------------------------------------------------------
// Structures

typedef struct
{
    DWORD   dwFlag;
    TCHAR * szFlagText;
    BOOL    bEnabled;

} TFlagInfo;

typedef struct
{
    TCHAR         szDirName[MAX_NT_PATH];
    TCHAR         szFileName1[MAX_NT_PATH];
    TCHAR         szFileName2[MAX_NT_PATH];
    HANDLE        hDirectory;
    HANDLE        hFile;
    HANDLE        hMap;                  // Map handle (if the file is mapped)
    PVOID         pvFile;                // Map pointer (if the file is mapped and accessed)
    DWORD         dwMapped;              // Number of bytes mapped
    ACCESS_MASK   dwDesiredAccess;
    LARGE_INTEGER liAllocationSize;
    ULONG         dwFileAttributes;
    ULONG         dwShareAccess;
    ULONG         dwCreateDisposition;
    ULONG         dwCreateOptions;
    ULONG         dwObjAttrFlags;        // ObjAttr.Attributes

} TPropSheetData;

typedef struct
{
    HWND        hParent;                 // Parent of the dialog
    UINT        nIDTitle;                // ID of the dialog title
    DWORD       dwFlags;                 // Flag value (in/out)
    int         nFlags;                  // Number of flags
    TFlagInfo * Flags;                   // Flags (structure array)
} TFlagDialogData;

//-----------------------------------------------------------------------------
// Global variables

extern HINSTANCE g_hInst;
extern HANDLE g_hHeap;

//-----------------------------------------------------------------------------
// NTSTATUS conversion

const TCHAR * NtStatus2Text(NTSTATUS Status);

//-----------------------------------------------------------------------------
// Utilities (in Utils.cpp)

DWORD Text2CreateDisposition(const TCHAR * szText);
void  CreateDisposition2Text(DWORD dwCreateDisposition, TCHAR * szBuffer);

DWORD DlgText2Hex32(HWND hDlg, UINT nIDCtrl);
DWORD Text2Hex32(const TCHAR * szText);
void  Hex2Text32(DWORD dwValue, TCHAR * szBuffer);
void  Hex2DlgText32(HWND hDlg, UINT nIDCtrl, DWORD dwValue);

DWORD_PTR DlgText2HexPtr(HWND hDlg, UINT nIDCtrl);
DWORD_PTR Text2HexPtr(const TCHAR * szText);
void  Hex2TextPtr(DWORD_PTR & dwpValue, TCHAR * szBuffer);
void Hex2DlgTextPtr(HWND hDlg, UINT nIDCtrl, DWORD_PTR dwpValue);

LARGE_INTEGER DlgText2Hex64(HWND hDlg, UINT nIDCtrl);
LARGE_INTEGER Text2Hex64(const TCHAR * szText);
void  Hex2Text64(LARGE_INTEGER & liValue, TCHAR * szBuffer);
void  Hex2DlgText64(HWND hDlg, UINT nIDCtrl, LARGE_INTEGER & liValue);

//int BrowseForFile(HWND hDlg, UINT nIDDirEdit, UINT nIDFileEdit);

int SetLastErrorText(HWND hDlg, UINT nIDCtrl, int nError);

//-----------------------------------------------------------------------------
// Global functions

INT_PTR FlagsDialog(TFlagDialogData * pData);

//-----------------------------------------------------------------------------
// Message handlers for each page

INT_PTR CALLBACK PageProc01(HWND hDlg, UINT uMsg, WPARAM wParam, LPARAM lParam);
INT_PTR CALLBACK PageProc02(HWND hDlg, UINT uMsg, WPARAM wParam, LPARAM lParam);
INT_PTR CALLBACK PageProc03(HWND hDlg, UINT uMsg, WPARAM wParam, LPARAM lParam);
INT_PTR CALLBACK PageProc04(HWND hDlg, UINT uMsg, WPARAM wParam, LPARAM lParam);
INT_PTR CALLBACK PageProc05(HWND hDlg, UINT uMsg, WPARAM wParam, LPARAM lParam);
INT_PTR CALLBACK PageProc06(HWND hDlg, UINT uMsg, WPARAM wParam, LPARAM lParam);

#endif // __TESTFILE_H__
