/*****************************************************************************/
/* Ntdll.cpp                              Copyright (c) Ladislav Zezula 2005 */
/*---------------------------------------------------------------------------*/
/* Wrapper for some Ntdll functions. Instead of statically linking Ntdll.dll,*/
/* I decided to write this simple wrapper module which causes using Ntdll.dll*/
/* functions comfortable to use like if the project linked Ntdll.lib         */
/*---------------------------------------------------------------------------*/
/*   Date    Ver   Who  Comment                                              */
/* --------  ----  ---  -------                                              */
/* 03.06.05  1.00  Lad  The first version of Ntdll.cpp                       */
/*****************************************************************************/

#include "FileTest.h"

//-----------------------------------------------------------------------------
// General-purpose function resolver

static NTSTATUS EnsureFunctionIsResolved(FARPROC & pFnc, LPCSTR szProcName)
{
    HINSTANCE hNtdll = NULL;
    NTSTATUS Status = STATUS_SUCCESS;

    // If the function pointer is already resolved, do nothing
    if(pFnc == NULL)
    {
        // Get the handle of Ntdll.dll and address of the entry point
        hNtdll = GetModuleHandle(_T("Ntdll.dll"));
        if(hNtdll != NULL)
            pFnc = GetProcAddress(hNtdll, szProcName);

        // If the function is still not resolved, it's bad.
        if(pFnc == NULL)
            Status = STATUS_ENTRYPOINT_NOT_FOUND;
    }

    return Status;
}


//-----------------------------------------------------------------------------
// NtCreateFile

NTSTATUS (WINAPI * pNtCreateFile)(PHANDLE, ACCESS_MASK, POBJECT_ATTRIBUTES, PIO_STATUS_BLOCK, PLARGE_INTEGER, ULONG, ULONG, ULONG, ULONG, PVOID, ULONG) = NULL;

extern "C"
NTSTATUS
WINAPI
NtCreateFile(
    OUT PHANDLE FileHandle,
    IN  ACCESS_MASK DesiredAccess,
    IN  POBJECT_ATTRIBUTES ObjectAttributes,
    OUT PIO_STATUS_BLOCK IoStatusBlock,
    IN  PLARGE_INTEGER AllocationSize,
    IN  ULONG FileAttributes,
    IN  ULONG ShareAccess,
    IN  ULONG CreateDisposition,
    IN  ULONG CreateOptions,
    IN  PVOID EaBuffer,
    IN  ULONG EaLength)
{
    NTSTATUS Status = EnsureFunctionIsResolved((FARPROC &)pNtCreateFile, "NtCreateFile");

    if(NT_SUCCESS(Status))
    {
        Status = pNtCreateFile(FileHandle,
                               DesiredAccess,
                               ObjectAttributes,
                               IoStatusBlock,
                               AllocationSize,
                               FileAttributes,
                               ShareAccess,
                               CreateDisposition,
                               CreateOptions,
                               EaBuffer,
                               EaLength);
    }

    return Status;
}

//-----------------------------------------------------------------------------
// NtQueryInformationFile

NTSTATUS (WINAPI * pNtQueryInformationFile)(HANDLE, PIO_STATUS_BLOCK, PVOID, ULONG, FILE_INFORMATION_CLASS) = NULL;

extern "C"
NTSTATUS
WINAPI
NtQueryInformationFile(
    IN HANDLE FileHandle,
    OUT PIO_STATUS_BLOCK IoStatusBlock,
    OUT PVOID FileInformation,
    IN ULONG Length,
    IN FILE_INFORMATION_CLASS FileInformationClass)
{
    NTSTATUS Status = EnsureFunctionIsResolved((FARPROC &)pNtQueryInformationFile, "NtQueryInformationFile");

    if(NT_SUCCESS(Status))
        Status = pNtQueryInformationFile(FileHandle, IoStatusBlock, FileInformation, Length, FileInformationClass);

    return Status;
}


//-----------------------------------------------------------------------------
// NtClose

NTSTATUS (WINAPI * pNtClose)(HANDLE) = NULL;

extern "C"
NTSTATUS
WINAPI
NtClose(
    IN  HANDLE Handle)
{
    NTSTATUS Status = EnsureFunctionIsResolved((FARPROC &)pNtClose, "NtClose");

    if(NT_SUCCESS(Status))
        Status = pNtClose(Handle);

    return Status;
}


//-----------------------------------------------------------------------------
// RtlDosPathNameToNtPathName_U

BOOLEAN (WINAPI * pRtlDosPathNameToNtPathName_U)(PWSTR, PUNICODE_STRING, PWSTR *, PCURDIR) = NULL;

extern "C"           
BOOLEAN
WINAPI
RtlDosPathNameToNtPathName_U(
    IN  PWSTR DosPathName,
    OUT PUNICODE_STRING NtPathName,
    OUT PWSTR * NtFileNamePart OPTIONAL,
    OUT PCURDIR DirectoryInfo OPTIONAL)
{
    NTSTATUS Status = EnsureFunctionIsResolved((FARPROC &)pRtlDosPathNameToNtPathName_U, "RtlDosPathNameToNtPathName_U");

    if(NT_SUCCESS(Status))
        return pRtlDosPathNameToNtPathName_U(DosPathName, NtPathName, NtFileNamePart, DirectoryInfo);

    return FALSE;
}
