/*****************************************************************************/
/* Page04FileOps.cpp                      Copyright (c) Ladislav Zezula 2004 */
/*---------------------------------------------------------------------------*/
/* Description:                                                              */
/*---------------------------------------------------------------------------*/
/*   Date    Ver   Who  Comment                                              */
/* --------  ----  ---  -------                                              */
/* 19.04.04  1.00  Lad  The first version of Page04FileOps.cpp               */
/*****************************************************************************/

#include "FileTest.h"
#include "resource.h"

//-----------------------------------------------------------------------------
// Helper functions

extern TFlagInfo FileAttributesValues[];

static BOOL CopyFileByHand(const TCHAR * szOrigFile, const TCHAR * szNewFile)
{
    FILETIME ft1;
    FILETIME ft2;
    FILETIME ft3;
    HANDLE hFile1 = INVALID_HANDLE_VALUE;
    HANDLE hFile2 = INVALID_HANDLE_VALUE;
    int nError = ERROR_SUCCESS;

    // Open the original file
    if(nError == ERROR_SUCCESS)
    {
        hFile1 = CreateFile(szOrigFile, GENERIC_READ, 0, NULL, OPEN_EXISTING, 0, NULL);
        if(IsHandleInvalid(hFile1))
            nError = GetLastError();
    }

    // (Re)create the new file
    if(nError == ERROR_SUCCESS)
    {
        hFile2 = CreateFile(szNewFile, GENERIC_READ | GENERIC_WRITE | DELETE | WRITE_DAC, 0, NULL, OPEN_ALWAYS, 0, NULL);
        if(IsHandleInvalid(hFile2))
            nError = GetLastError();
    }

    // Get the file time of the original file
    if(nError == ERROR_SUCCESS)
    {
        if(!GetFileTime(hFile1, &ft1, &ft2, &ft3))
            nError = GetLastError();
    }

    // Copy the content
    if(nError == ERROR_SUCCESS)
    {
        BYTE  * pbBuffer = NULL;
        DWORD dwBufferSize = 0x10000;
        DWORD dwTransferred = 1;

        pbBuffer = new BYTE [dwBufferSize];
        while(dwTransferred != 0)
        {
            ReadFile(hFile1, pbBuffer, dwBufferSize, &dwTransferred, NULL);
            if(dwTransferred != 0)
                WriteFile(hFile2, pbBuffer, dwTransferred, &dwTransferred, NULL);
        }
        delete pbBuffer;
    }

    // Get the file time of the original file
    if(nError == ERROR_SUCCESS)
    {
        if(!SetFileTime(hFile2, &ft1, &ft2, &ft3))
            nError = GetLastError();
    }

    // Close both files
    if(IsHandleValid(hFile2))
        CloseHandle(hFile2);
    if(IsHandleValid(hFile1))
        CloseHandle(hFile1);
    
    if(nError != ERROR_SUCCESS)
    {
        SetLastError(nError);
        return FALSE;
    }
    return TRUE;
}

static int SaveDialog(HWND hDlg)
{
    TPropSheetData * pData = (TPropSheetData *)GetWindowLongPtr(hDlg, DWLP_USER);

    if((pData->dwCreateOptions & FILE_OPEN_BY_FILE_ID) == 0)
        GetDlgItemText(hDlg, IDC_FILE_NAME1, pData->szFileName1, _tsize(pData->szFileName1));
    GetDlgItemText(hDlg, IDC_FILE_NAME2, pData->szFileName2, _tsize(pData->szFileName2));
    return ERROR_SUCCESS;
}


// This function enables/disables the buttons for map operations
static int UpdateMapFunctions(HWND hDlg)
{
    TPropSheetData * pData = (TPropSheetData *)GetWindowLongPtr(hDlg, DWLP_USER);

    EnableWindow(GetDlgItem(hDlg, IDC_MAP_FILE), IsHandleValid(pData->hFile));
    EnableWindow(GetDlgItem(hDlg, IDC_ACCESS_DATA), (pData->pvFile != NULL));
    EnableWindow(GetDlgItem(hDlg, IDC_UNMAP_FILE), (pData->pvFile != NULL || IsHandleValid(pData->hMap)));
    return TRUE;
}


//-----------------------------------------------------------------------------
// Message handlers

static int OnInitDialog(HWND hDlg, LPARAM lParam)
{
    PROPSHEETPAGE * pPage = (PROPSHEETPAGE *)lParam;
    LARGE_INTEGER LargeZero = {0};

    SetWindowLong(hDlg, DWLP_USER, (LONG)pPage->lParam);
    Button_SetCheck(GetDlgItem(hDlg, IDC_USE_COPYAPI), BST_CHECKED);

    Hex2DlgText64(hDlg, IDC_FILE_POSITION, LargeZero);
    Hex2DlgText32(hDlg, IDC_DATA_LENGTH, 0x10000);
    return TRUE;
}

static int OnSetActive(HWND hDlg)
{
    TPropSheetData * pData = (TPropSheetData *)GetWindowLongPtr(hDlg, DWLP_USER);
    TCHAR * szName;

    if((pData->dwCreateOptions & FILE_OPEN_BY_FILE_ID) == 0)
    {
        szName = pData->szFileName1;
        if(!_tcsncmp(szName, _T("\\??\\"), 4))
            szName += 4;
        SetDlgItemText(hDlg, IDC_FILE_NAME1, szName);
    }
    else
    {
        pData->szDirName[0] = 0;
        pData->dwCreateOptions &= ~FILE_OPEN_BY_FILE_ID;
    }

    if(pData->szFileName2[0] != 0)
        SetDlgItemText(hDlg, IDC_FILE_NAME2, pData->szFileName2);

    UpdateMapFunctions(hDlg);
    return TRUE;
}

static int OnKillActive(HWND hDlg)
{
    SaveDialog(hDlg);
    UpdateMapFunctions(hDlg);
    return TRUE;
}

static int OnCopyFileClick(HWND hDlg)
{
    TPropSheetData * pData = (TPropSheetData *)GetWindowLongPtr(hDlg, DWLP_USER);
    BOOL  bUseCopyFile = FALSE;
    BOOL  bResult = FALSE;
    int nError = ERROR_SUCCESS;

    // Decide whether we have to copy by hand or not
    SaveDialog(hDlg);
    bUseCopyFile = (Button_GetCheck(GetDlgItem(hDlg, IDC_USE_COPYAPI)) == BST_CHECKED);

    // Perform the copy
    if(nError == ERROR_SUCCESS)
    {
        GetDlgItemText(hDlg, IDC_FILE_NAME1, pData->szFileName1, _tsize(pData->szFileName1));
        GetDlgItemText(hDlg, IDC_FILE_NAME2, pData->szFileName2, _tsize(pData->szFileName2));

        if(bUseCopyFile)
            bResult = CopyFile(pData->szFileName1, pData->szFileName2, FALSE);
        else
            bResult = CopyFileByHand(pData->szFileName1, pData->szFileName2);

        if(bResult == FALSE)
            nError = GetLastError();
    }

    // Set the result
    SetLastErrorText(hDlg, IDC_LAST_ERROR, nError);
    return TRUE;
}

static int OnMoveFileClick(HWND hDlg)
{
    TPropSheetData * pData = (TPropSheetData *)GetWindowLongPtr(hDlg, DWLP_USER);
    DWORD dwMoveOptions = MOVEFILE_COPY_ALLOWED;
    int nError = ERROR_SUCCESS;

    SaveDialog(hDlg);

    if(IsDlgButtonChecked(hDlg, IDC_REPLACE_EXISTING))
        dwMoveOptions |= MOVEFILE_REPLACE_EXISTING;

    // Perform the rename
    if(!MoveFileEx(pData->szFileName1, pData->szFileName2, dwMoveOptions))
        nError = GetLastError();

    // Set the result
    SetLastErrorText(hDlg, IDC_LAST_ERROR, nError);
    return TRUE;
}

static int OnMapFileClick(HWND hDlg)
{
    TPropSheetData * pData = (TPropSheetData *)GetWindowLongPtr(hDlg, DWLP_USER);
    LARGE_INTEGER StartOffset = {0};
    LARGE_INTEGER MaxMapSize;
    DWORD dwFileSize = GetFileSize(pData->hFile, NULL);
    int nError = ERROR_SUCCESS;

    // Create the file mapping object
    if(nError == ERROR_SUCCESS)
    {
        StartOffset     = DlgText2Hex64(hDlg, IDC_FILE_POSITION);
        pData->dwMapped = DlgText2Hex32(hDlg, IDC_DATA_LENGTH);

        // Get the maximum map size
        MaxMapSize.QuadPart = dwFileSize;
        MaxMapSize.QuadPart -= StartOffset.QuadPart;
        if(pData->dwMapped > MaxMapSize.QuadPart)
            pData->dwMapped = (ULONG)MaxMapSize.QuadPart;

        if(IsHandleInvalid(pData->hMap))
            pData->hMap = CreateFileMapping(pData->hFile, NULL, PAGE_READWRITE, 0, pData->dwMapped, NULL);

        if(IsHandleInvalid(pData->hMap))
            nError = GetLastError();
    }

    // Map view of file
    if(nError == ERROR_SUCCESS)
    {
        if(pData->pvFile == NULL)
            pData->pvFile = MapViewOfFile(pData->hMap, FILE_MAP_WRITE, StartOffset.HighPart, StartOffset.LowPart, pData->dwMapped);
        if(pData->pvFile == NULL)
            nError = GetLastError();
    }

    UpdateMapFunctions(hDlg);
    SetLastErrorText(hDlg, IDC_LAST_ERROR, nError);
    return TRUE;
}

static int OnAccessDataClick(HWND hDlg)
{
    TPropSheetData * pData = (TPropSheetData *)GetWindowLongPtr(hDlg, DWLP_USER);
    PBYTE pbDataBegin = (PBYTE)pData->pvFile;
    PBYTE pbDataEnd = (PBYTE)pData->pvFile + pData->dwMapped;
    int  nSum = 0;

    if(pData->pvFile != NULL && pData->dwMapped > 0)
    {
        while(pbDataBegin < pbDataEnd)
        {
            nSum += *pbDataBegin++;
        }
    }
    return TRUE;
}

static int OnUnmapFileClick(HWND hDlg)
{
    TPropSheetData * pData = (TPropSheetData *)GetWindowLongPtr(hDlg, DWLP_USER);
    int nError = ERROR_SUCCESS;

    if(pData->pvFile != NULL)
    {
        UnmapViewOfFile(pData->pvFile);
        pData->pvFile = NULL;
    }

    if(IsHandleValid(pData->hMap))
    {
        CloseHandle(pData->hMap);
        pData->hMap = NULL;
    }

    UpdateMapFunctions(hDlg);
    SetLastErrorText(hDlg, IDC_LAST_ERROR, nError);
    return TRUE;
}

static int OnDeleteFileClick(HWND hDlg)
{
    TPropSheetData * pData = (TPropSheetData *)GetWindowLongPtr(hDlg, DWLP_USER);
    int nError = ERROR_SUCCESS;

    SaveDialog(hDlg);
    if(!DeleteFile(pData->szFileName1))
        nError = GetLastError();

    UpdateMapFunctions(hDlg);
    SetLastErrorText(hDlg, IDC_LAST_ERROR, nError);
    return TRUE;
}

// Querying the file ID from the file itself
static int OnGetFileIdClick(HWND hDlg)
{
    TPropSheetData * pData = (TPropSheetData *)GetWindowLongPtr(hDlg, DWLP_USER);
    FILE_INTERNAL_INFORMATION FileInternalInfo = {0};
    OBJECT_ATTRIBUTES ObjAttr;
    IO_STATUS_BLOCK IoStatus;
    UNICODE_STRING FileName = {0, 0, NULL};
    NTSTATUS Status = STATUS_SUCCESS;
    HANDLE FileHandle = NULL;
    TCHAR szFileID[32];

    // Convert the file name to the NT file name
    if(NT_SUCCESS(Status))
    {
        SaveDialog(hDlg);
        if(!RtlDosPathNameToNtPathName_U(pData->szFileName1, &FileName, NULL, NULL))
            Status = STATUS_UNSUCCESSFUL;
    }

    // Open the directory
    if(NT_SUCCESS(Status))
    {
        InitializeObjectAttributes(&ObjAttr, &FileName, OBJ_CASE_INSENSITIVE, NULL, NULL);
        Status = NtCreateFile(&FileHandle,
                               FILE_READ_ATTRIBUTES,
                              &ObjAttr,
                              &IoStatus,
                               NULL,
                               0,
                               FILE_SHARE_READ | FILE_SHARE_WRITE | FILE_SHARE_DELETE,
                               FILE_OPEN,
                               0,
                               NULL,
                               0);
    }

    // Query the file internal information
    if(NT_SUCCESS(Status))
    {
        Status = NtQueryInformationFile(FileHandle,
                                       &IoStatus,
                                       &FileInternalInfo,
                                        sizeof(FILE_INTERNAL_INFORMATION),
                                        FileInternalInformation);
    }

    if(NT_SUCCESS(Status))
    {
        _stprintf(szFileID, _T("%c:\\%016I64x"), pData->szFileName1[0], FileInternalInfo.IndexNumber.QuadPart);
        SetDlgItemText(hDlg, IDC_FILE_ID, szFileID);
    }

    // On the end, set the file ID
    if(FileHandle != NULL)
        NtClose(FileHandle);
    if(FileName.Buffer != NULL)
        HeapFree(GetProcessHeap(), 0, FileName.Buffer);
    SetDlgItemText(hDlg, IDC_LAST_ERROR, NtStatus2Text(Status));
    return TRUE;
}

static int OnFileIDChange(HWND hDlg)
{
    HWND hEdit = GetDlgItem(hDlg, IDC_FILE_ID);
    BOOL bEnable = (GetWindowTextLength(hEdit) != 0);

    EnableWindow(GetDlgItem(hDlg, IDC_USE_FILE_ID), bEnable);
    return TRUE;
}

static int OnUseFileId(HWND hDlg)
{
    TPropSheetData * pData = (TPropSheetData *)GetWindowLongPtr(hDlg, DWLP_USER);
    LPTSTR szPlainName;
    HWND hPropSheet = GetParent(hDlg);

    // Setup the create options for opening by file ID
    GetDlgItemText(hDlg, IDC_FILE_ID, pData->szDirName, _tsize(pData->szDirName));
    szPlainName = GetPlainName(pData->szDirName);
    _tcscpy(pData->szFileName1, szPlainName);
    *szPlainName = 0;
    pData->dwCreateOptions = FILE_OPEN_BY_FILE_ID;

    PropSheet_SetCurSelByID(hPropSheet, IDD_PAGE05_NTCREATE);
    return TRUE;
}

static int OnGetFileAttributes(HWND hDlg)
{
    TPropSheetData * pData = (TPropSheetData *)GetWindowLongPtr(hDlg, DWLP_USER);
    TCHAR szFileAttributes[512] = _T("");
    DWORD dwAttr;
    int nError = ERROR_SUCCESS;

    SaveDialog(hDlg);
    dwAttr = GetFileAttributes(pData->szFileName1);
    if(dwAttr != INVALID_FILE_ATTRIBUTES)
    {
        for(int i = 0; FileAttributesValues[i].dwFlag != 0; i++)
        {
            if(dwAttr & FileAttributesValues[i].dwFlag)
            {
                _tcscat(szFileAttributes, FileAttributesValues[i].szFlagText);
                _tcscat(szFileAttributes, _T("\n"));
            }
        }

        if(szFileAttributes[0] == 0)
            _tcscpy(szFileAttributes, _T("0"));
        MessageBoxRc(hDlg, IDS_FILE_ATTRIBUTES, (UINT_PTR)szFileAttributes);
    }
    else
        nError = GetLastError();

    SetLastErrorText(hDlg, IDC_LAST_ERROR, nError);
    return TRUE;
}

static int OnCommand(HWND hDlg, UINT nNotify, UINT nIDCtrl)
{
    if(nNotify == BN_CLICKED)
    {
        switch(nIDCtrl)
        {
            case IDC_BROWSE_FILE1:
                BrowseForFile(hDlg, NULL, MAKEINTRESOURCE(IDC_FILE_NAME1), IDS_SELECT_FILE, IDS_FILTER_ALL);
                return TRUE;

            case IDC_BROWSE_FILE2:
                BrowseForFile(hDlg, NULL, MAKEINTRESOURCE(IDC_FILE_NAME2), IDS_SELECT_FILE, IDS_FILTER_ALL);
                return TRUE;

            case IDC_COPYFILE:
                return OnCopyFileClick(hDlg);

            case IDC_MOVEFILE:
                return OnMoveFileClick(hDlg);

            case IDC_DELETE_FILE:
                return OnDeleteFileClick(hDlg);

            case IDC_MAP_FILE:
                return OnMapFileClick(hDlg);

            case IDC_ACCESS_DATA:
                return OnAccessDataClick(hDlg);

            case IDC_UNMAP_FILE:
                return OnUnmapFileClick(hDlg);

            case IDC_GET_FILEID:
                return OnGetFileIdClick(hDlg);

            case IDC_USE_FILE_ID:
                return OnUseFileId(hDlg);

            case IDC_GET_FILE_ATTRIBUTES:
                return OnGetFileAttributes(hDlg);
        }
    }

    if(nNotify == EN_CHANGE && nIDCtrl == IDC_FILE_ID)
        return OnFileIDChange(hDlg);

    return FALSE;
}

static int OnNotify(HWND hDlg, NMHDR * pNMHDR)
{
    switch(pNMHDR->code)
    {
        case PSN_SETACTIVE:
            return OnSetActive(hDlg);

        case PSN_KILLACTIVE:
            return OnKillActive(hDlg);

//      case UDN_DELTAPOS:
//          return OnDeltaPos(hDlg, (NMUPDOWN *)pNMHDR);
    }
    return FALSE;
}

//-----------------------------------------------------------------------------
// Public functions

INT_PTR CALLBACK PageProc04(HWND hDlg, UINT uMsg, WPARAM wParam, LPARAM lParam)
{
    switch(uMsg)
    {
        case WM_INITDIALOG:
            return OnInitDialog(hDlg, lParam);

        case WM_COMMAND:
            return OnCommand(hDlg, HIWORD(wParam), LOWORD(wParam));

        case WM_NOTIFY:
            return OnNotify(hDlg, (NMHDR *)lParam);
    }
    return FALSE;
}
