/*****************************************************************************/
/* Page05NtCreate.cpp                     Copyright (c) Ladislav Zezula 2004 */
/*---------------------------------------------------------------------------*/
/* Description:                                                              */
/*---------------------------------------------------------------------------*/
/*   Date    Ver   Who  Comment                                              */
/* --------  ----  ---  -------                                              */
/* 06.03.04  1.00  Lad  The first version of Page05NtCreate.cpp              */
/*****************************************************************************/

#include "FileTest.h"
#include "resource.h"

#define FILE_ID_SIZE 16

//-----------------------------------------------------------------------------
// Flags

static TFlagInfo DesiredAccessValues[] =
{
    {FILE_READ_DATA,         _T("FILE_READ_DATA/FILE_LIST_DIRECTORY"),     TRUE},
    {FILE_WRITE_DATA,        _T("FILE_WRITE_DATA/FILE_ADD_FILE"),          TRUE},
    {FILE_APPEND_DATA,       _T("FILE_APPEND_DATA/FILE_ADD_SUBDIRECTORY"), TRUE},
    {FILE_READ_EA,           _T("FILE_READ_EA"),                           TRUE},
    {FILE_WRITE_EA,          _T("FILE_WRITE_EA"),                          TRUE},
    {FILE_EXECUTE,           _T("FILE_EXECUTE/FILE_TRAVERSE"),             TRUE},
    {FILE_DELETE_CHILD,      _T("FILE_DELETE_CHILD"),                      TRUE},
    {FILE_READ_ATTRIBUTES,   _T("FILE_READ_ATTRIBUTES"),                   TRUE},
    {FILE_WRITE_ATTRIBUTES,  _T("FILE_WRITE_ATTRIBUTES"),                  TRUE},
    {DELETE,                 _T("DELETE"),                                 TRUE},
    {READ_CONTROL,           _T("READ_CONTROL"),                           TRUE},
    {WRITE_DAC,              _T("WRITE_DAC"),                              TRUE},
    {WRITE_OWNER,            _T("WRITE_OWNER"),                            TRUE},
    {SYNCHRONIZE,            _T("SYNCHRONIZE"),                            TRUE},
    {GENERIC_READ,           _T("GENERIC_READ"),                           TRUE},
    {GENERIC_WRITE,          _T("GENERIC_WRITE"),                          TRUE},
    {GENERIC_EXECUTE,        _T("GENERIC_EXECUTE"),                        TRUE},
    {GENERIC_ALL,            _T("GENERIC_ALL"),                            TRUE},
    {0, 0, 0}
};

// Also necessary in Page04FileOps.cpp
TFlagInfo FileAttributesValues[] =
{
    {FILE_ATTRIBUTE_READONLY,            _T("FILE_ATTRIBUTE_READONLY"),            TRUE},
    {FILE_ATTRIBUTE_HIDDEN,              _T("FILE_ATTRIBUTE_HIDDEN"),              TRUE},
    {FILE_ATTRIBUTE_SYSTEM,              _T("FILE_ATTRIBUTE_SYSTEM"),              TRUE},
    {OLD_DOS_VOLID,                      _T("OLD_DOS_VOLID"),                      FALSE},
    {FILE_ATTRIBUTE_DIRECTORY,           _T("FILE_ATTRIBUTE_DIRECTORY"),           TRUE},
    {FILE_ATTRIBUTE_ARCHIVE,             _T("FILE_ATTRIBUTE_ARCHIVE"),             TRUE},
    {FILE_ATTRIBUTE_DEVICE,              _T("FILE_ATTRIBUTE_DEVICE"),              TRUE},
    {FILE_ATTRIBUTE_NORMAL,              _T("FILE_ATTRIBUTE_NORMAL"),              TRUE},
    {FILE_ATTRIBUTE_TEMPORARY,           _T("FILE_ATTRIBUTE_TEMPORARY"),           TRUE},
    {FILE_ATTRIBUTE_SPARSE_FILE,         _T("FILE_ATTRIBUTE_SPARSE_FILE"),         TRUE},
    {FILE_ATTRIBUTE_REPARSE_POINT,       _T("FILE_ATTRIBUTE_REPARSE_POINT"),       TRUE},
    {FILE_ATTRIBUTE_COMPRESSED,          _T("FILE_ATTRIBUTE_COMPRESSED"),          TRUE},
    {FILE_ATTRIBUTE_OFFLINE,             _T("FILE_ATTRIBUTE_OFFLINE"),             TRUE},
    {FILE_ATTRIBUTE_NOT_CONTENT_INDEXED, _T("FILE_ATTRIBUTE_NOT_CONTENT_INDEXED"), TRUE},
    {FILE_ATTRIBUTE_ENCRYPTED,           _T("FILE_ATTRIBUTE_ENCRYPTED"),           TRUE},
    {0, 0, 0}
};

static TFlagInfo ShareAccessValues[] =
{
    {FILE_SHARE_READ,   _T("FILE_SHARE_READ"),   TRUE},
    {FILE_SHARE_WRITE,  _T("FILE_SHARE_WRITE"),  TRUE},
    {FILE_SHARE_DELETE, _T("FILE_SHARE_DELETE"), TRUE},
    {0, 0, 0}
};

static TFlagInfo CreateOptionsValues[] =
{
    {FILE_DIRECTORY_FILE,            _T("FILE_DIRECTORY_FILE"),            TRUE},
    {FILE_WRITE_THROUGH,             _T("FILE_WRITE_THROUGH"),             TRUE},
    {FILE_SEQUENTIAL_ONLY,           _T("FILE_SEQUENTIAL_ONLY"),           TRUE},
    {FILE_NO_INTERMEDIATE_BUFFERING, _T("FILE_NO_INTERMEDIATE_BUFFERING"), TRUE},
    {FILE_SYNCHRONOUS_IO_ALERT,      _T("FILE_SYNCHRONOUS_IO_ALERT"),      TRUE},
    {FILE_SYNCHRONOUS_IO_NONALERT,   _T("FILE_SYNCHRONOUS_IO_NONALERT"),   TRUE},
    {FILE_NON_DIRECTORY_FILE,        _T("FILE_NON_DIRECTORY_FILE"),        TRUE},
    {FILE_CREATE_TREE_CONNECTION,    _T("FILE_CREATE_TREE_CONNECTION"),    TRUE},
    {FILE_COMPLETE_IF_OPLOCKED,      _T("FILE_COMPLETE_IF_OPLOCKED"),      TRUE},
    {FILE_NO_EA_KNOWLEDGE,           _T("FILE_NO_EA_KNOWLEDGE"),           TRUE},
    {FILE_OPEN_FOR_RECOVERY,         _T("FILE_OPEN_FOR_RECOVERY"),         TRUE},
    {FILE_RANDOM_ACCESS,             _T("FILE_RANDOM_ACCESS"),             TRUE},
    {FILE_DELETE_ON_CLOSE,           _T("FILE_DELETE_ON_CLOSE"),           TRUE},
    {FILE_OPEN_BY_FILE_ID,           _T("FILE_OPEN_BY_FILE_ID"),           TRUE},
    {FILE_OPEN_FOR_BACKUP_INTENT,    _T("FILE_OPEN_FOR_BACKUP_INTENT"),    TRUE},
    {FILE_NO_COMPRESSION,            _T("FILE_NO_COMPRESSION"),            TRUE},
    {FILE_RESERVE_OPFILTER,          _T("FILE_RESERVE_OPFILTER"),          TRUE},
    {FILE_OPEN_REPARSE_POINT,        _T("FILE_OPEN_REPARSE_POINT"),        TRUE},
    {FILE_OPEN_NO_RECALL,            _T("FILE_OPEN_NO_RECALL"),            TRUE},
    {FILE_OPEN_FOR_FREE_SPACE_QUERY, _T("FILE_OPEN_FOR_FREE_SPACE_QUERY"), TRUE},
    {0, 0, 0}
};

static TFlagInfo ObjAttrFlagsValues[] =
{
    {OBJ_INHERIT,                    _T("OBJ_INHERIT"),                    TRUE},
    {OBJ_PERMANENT,                  _T("OBJ_PERMANENT"),                  TRUE},
    {OBJ_EXCLUSIVE,                  _T("OBJ_EXCLUSIVE"),                  TRUE},
    {OBJ_CASE_INSENSITIVE,           _T("OBJ_CASE_INSENSITIVE"),           TRUE},    
    {OBJ_OPENIF,                     _T("OBJ_OPENIF"),                     TRUE},
    {OBJ_OPENLINK,                   _T("OBJ_OPENLINK"),                   TRUE},
    {OBJ_KERNEL_HANDLE,              _T("OBJ_KERNEL_HANDLE"),              TRUE},
    {OBJ_FORCE_ACCESS_CHECK,         _T("OBJ_FORCE_ACCESS_CHECK"),         TRUE},
    {0, 0, 0}                        
};

//-----------------------------------------------------------------------------
// Local functions

static void SetNtOperationResult(HWND hDlg, NTSTATUS Status, PIO_STATUS_BLOCK pIoStatus)
{
    TPropSheetData * pData = (TPropSheetData *)GetWindowLongPtr(hDlg, DWLP_USER);
    BOOL bEnable = TRUE;

    TCHAR * szIoStatus[] = 
    {
        _T("FILE_SUPERSEDED"),
        _T("FILE_OPENED"),
        _T("FILE_CREATED"),
        _T("FILE_OVERWRITTEN"),
        _T("FILE_EXISTS"),
        _T("FILE_DOES_NOT_EXIST")
    };

    SetDlgItemText(hDlg, IDC_RESULT_STATUS, NtStatus2Text(Status));
    Hex2DlgTextPtr(hDlg, IDC_FILE_HANDLE, (DWORD_PTR)pData->hFile);

    // Set the IO status
    if(Status == STATUS_SUCCESS && pIoStatus != NULL && pIoStatus->Information <= FILE_DOES_NOT_EXIST)
        SetDlgItemText(hDlg, IDC_IOSTATUS_RESULT, szIoStatus[pIoStatus->Information]);
    else
        SetDlgItemText(hDlg, IDC_IOSTATUS_RESULT, _T(""));

    // Enable NtClose only if the handle has been retrieved using NtCreateFile
    if(IsHandleInvalid(pData->hFile))
        bEnable = FALSE;
    EnableWindow(GetDlgItem(hDlg, IDC_CLOSEHINT), bEnable);
    EnableWindow(GetDlgItem(hDlg, IDC_NTCLOSE), bEnable);
}

static int ConvertStringToFileID(HWND hDlg, UINT nIDCtrl, UNICODE_STRING & FileName)
{
    LARGE_INTEGER FileId = {0};

    // Convert the value to 64bit integer and copy to the UNICODE_STRING
    FileId = DlgText2Hex64(hDlg, nIDCtrl);
    memcpy((PCHAR)FileName.Buffer, &FileId, sizeof(FileId));
    FileName.Length = sizeof(FileId);
    return ERROR_SUCCESS;
}

static int SaveDialog(HWND hDlg)
{
    TPropSheetData * pData = (TPropSheetData *)GetWindowLongPtr(hDlg, DWLP_USER);
    HWND hCombo = GetDlgItem(hDlg, IDC_CREATE_DISPOSITION);

    GetDlgItemText(hDlg, IDC_DIRECTORY_NAME, pData->szDirName, _tsize(pData->szDirName));
    GetDlgItemText(hDlg, IDC_FILE_NAME, pData->szFileName1, _tsize(pData->szFileName1));
    pData->dwDesiredAccess     = DlgText2Hex32(hDlg, IDC_DESIRED_ACCESS);
    pData->dwFileAttributes    = DlgText2Hex32(hDlg, IDC_FILE_ATTRIBUTES);
    pData->dwShareAccess       = DlgText2Hex32(hDlg, IDC_SHARE_ACCESS);
    pData->dwCreateOptions     = DlgText2Hex32(hDlg, IDC_CREATE_OPTIONS);
    pData->dwObjAttrFlags      = DlgText2Hex32(hDlg, IDC_OBJ_ATTR_FLAGS);
    pData->liAllocationSize    = DlgText2Hex64(hDlg, IDC_ALLOCATION_SIZE);
    pData->dwCreateDisposition = ComboBox_GetCurSel(hCombo);

    return ERROR_SUCCESS;
}
                                                             
//-----------------------------------------------------------------------------
// Message handlers

static int ConvertToNtName(HWND hDlg, UINT nIDEdit)
{
    UNICODE_STRING NtName = {0};
    NTSTATUS Status = STATUS_SUCCESS;
    HWND hEdit = GetDlgItem(hDlg, nIDEdit);
    TCHAR * szFileName = NULL;
    int nLength = GetWindowTextLength(hEdit) + 1;

    __try
    {
        if(nLength == 1)
            return STATUS_SUCCESS;

        // Allocate buffer
        szFileName = new TCHAR[nLength];
        if(szFileName == NULL)
            return ERROR_NOT_ENOUGH_MEMORY;

        // Get the window text
        // If it already appears to be an NT name, do nothing
        GetWindowText(hEdit, szFileName, nLength);
        if(szFileName[0] == 0 || szFileName[0] == _T('\\') || szFileName[1] != _T(':'))
            return ERROR_SUCCESS;

        Status = RtlDosPathNameToNtPathName_U(szFileName,
                                             &NtName,
                                              NULL,
                                              NULL);

        //
        // Maximum names that may be handled by RtlDosPathNameToNtPathName_U
        // is about 0x200 charascters.
        //

        if(!NT_SUCCESS(Status) || NtName.Length == 0)
        {
            if(isalpha(szFileName[0]) && szFileName[1] == _T(':') && szFileName[2] == _T('\\'))
            {
                NtName.MaximumLength = (USHORT)(nLength + 5);
                NtName.Length = (USHORT)(nLength + 4);
                NtName.Buffer = (PWSTR)HeapAlloc(g_hHeap, HEAP_ZERO_MEMORY, NtName.MaximumLength);
                _tcscpy(NtName.Buffer, _T("\\??\\"));
                _tcscat(NtName.Buffer, szFileName);
            }
        }

        // Set the NT name to the edit box
        if(NtName.Length != 0)
            SetWindowText(hEdit, NtName.Buffer);
    }
    __finally
    {
        if(NtName.Buffer != NULL)
            HeapFree(g_hHeap, 0, NtName.Buffer);
        if(szFileName != NULL)
            delete [] szFileName;
    }

    return Status;
}

static int OnBrowseDirClick(HWND hDlg)
{
    BrowseForDirectory(hDlg, IDC_DIRECTORY_NAME, IDS_SELECT_DIRECTORY);
    ConvertToNtName(hDlg, IDC_DIRECTORY_NAME);
    return TRUE;
}

static int OnBrowseFileClick(HWND hDlg)
{
    if(BrowseForFile(hDlg, 0, MAKEINTRESOURCE(IDC_FILE_NAME), IDS_SELECT_FILE, IDS_FILTER_ALL))
    {
        SetDlgItemText(hDlg, IDC_DIRECTORY_NAME, _T(""));
        ConvertToNtName(hDlg, IDC_FILE_NAME);
    }
    return TRUE;
}

static int OnDesiredAccessClick(HWND hDlg)
{
    TFlagDialogData DesiredAccessData;

    DesiredAccessData.hParent  = hDlg;
    DesiredAccessData.nIDTitle = IDS_DESIRED_ACCESS;
    DesiredAccessData.dwFlags  = DlgText2Hex32(hDlg, IDC_DESIRED_ACCESS);
    DesiredAccessData.nFlags   = sizeof(DesiredAccessValues) / sizeof(TFlagInfo);
    DesiredAccessData.Flags    = DesiredAccessValues;

    FlagsDialog(&DesiredAccessData);
    Hex2DlgText32(hDlg, IDC_DESIRED_ACCESS, DesiredAccessData.dwFlags);
    return TRUE;
}

static int OnFileAttributesClick(HWND hDlg)
{
    TFlagDialogData FileAttributesData;

    FileAttributesData.hParent  = hDlg;
    FileAttributesData.nIDTitle = IDS_FILE_ATTRIBUTES;
    FileAttributesData.dwFlags  = DlgText2Hex32(hDlg, IDC_FILE_ATTRIBUTES);
    FileAttributesData.nFlags   = sizeof(FileAttributesValues) / sizeof(TFlagInfo);
    FileAttributesData.Flags    = FileAttributesValues;

    FlagsDialog(&FileAttributesData);
    Hex2DlgText32(hDlg, IDC_FILE_ATTRIBUTES, FileAttributesData.dwFlags);
    return TRUE;
}

static int OnShareAccessClick(HWND hDlg)
{
    TFlagDialogData ShareAccessData;

    ShareAccessData.hParent  = hDlg;
    ShareAccessData.nIDTitle = IDS_SHARE_ACCESS;
    ShareAccessData.dwFlags  = DlgText2Hex32(hDlg, IDC_SHARE_ACCESS);
    ShareAccessData.nFlags   = sizeof(ShareAccessValues) / sizeof(TFlagInfo);
    ShareAccessData.Flags    = ShareAccessValues;

    FlagsDialog(&ShareAccessData);
    Hex2DlgText32(hDlg, IDC_SHARE_ACCESS, ShareAccessData.dwFlags);
    return TRUE;
}

static int OnCreateOptionsClick(HWND hDlg)
{
    TFlagDialogData CreateOptionsData;

    CreateOptionsData.hParent  = hDlg;
    CreateOptionsData.nIDTitle = IDS_CREATE_OPTIONS;
    CreateOptionsData.dwFlags  = DlgText2Hex32(hDlg, IDC_CREATE_OPTIONS);
    CreateOptionsData.nFlags   = sizeof(CreateOptionsValues) / sizeof(TFlagInfo);
    CreateOptionsData.Flags    = CreateOptionsValues;

    FlagsDialog(&CreateOptionsData);
    Hex2DlgText32(hDlg, IDC_CREATE_OPTIONS, CreateOptionsData.dwFlags);
    return TRUE;
}

static int OnObjAtributesFlags(HWND hDlg)
{
    TFlagDialogData ObjAttrFlagsData;

    ObjAttrFlagsData.hParent  = hDlg;
    ObjAttrFlagsData.nIDTitle = IDS_OBJECT_ATTRIBUTES_FLAGS;
    ObjAttrFlagsData.dwFlags  = DlgText2Hex32(hDlg, IDC_OBJ_ATTR_FLAGS);
    ObjAttrFlagsData.nFlags   = sizeof(CreateOptionsValues) / sizeof(TFlagInfo);
    ObjAttrFlagsData.Flags    = ObjAttrFlagsValues;

    FlagsDialog(&ObjAttrFlagsData);
    Hex2DlgText32(hDlg, IDC_OBJ_ATTR_FLAGS, ObjAttrFlagsData.dwFlags);
    return TRUE;
}

static int OnCreateDispositionChanged(HWND hDlg)
{
    DWORD dwCreateDisposition;
    DWORD dwCreateOptions = DlgText2Hex32(hDlg, IDC_CREATE_OPTIONS);
    HWND hCombo = GetDlgItem(hDlg, IDC_CREATE_DISPOSITION);

    dwCreateOptions &= 0x00FFFFFF;
    dwCreateDisposition = ComboBox_GetCurSel(hCombo);
    dwCreateOptions |= (dwCreateDisposition << 24);

    Hex2DlgText32(hDlg, IDC_CREATE_OPTIONS, dwCreateOptions);
    return TRUE;
}

static int OnOpenFileClick(HWND hDlg)
{
    TPropSheetData * pData = (TPropSheetData *)GetWindowLongPtr(hDlg, DWLP_USER);
    OBJECT_ATTRIBUTES ObjAttr;
    IO_STATUS_BLOCK IoStatus;
    UNICODE_STRING FileName;
    UNICODE_STRING DirName;
    NTSTATUS Status = STATUS_SUCCESS;

    if(IsHandleValid(pData->hFile))
    {
        MessageBoxRc(hDlg, IDS_WARNING, IDS_FILE_ALREADY_OPEN);
        return TRUE;
    }

    // Get the various create options
    SaveDialog(hDlg);

    // Id we required opening by ID and it is not there, do nothing
    if(pData->szDirName[0] == 0 && (pData->dwCreateOptions & FILE_OPEN_BY_FILE_ID))
    {
        MessageBoxRc(hDlg, IDS_ERROR, IDS_OPEN_BY_ID_HINT);
        return TRUE;
    }

    // Open the directory (If any)
    if(pData->szDirName[0] != 0)
    {
        INIT_UNICODE_STRING(DirName, pData->szDirName);

        InitializeObjectAttributes(&ObjAttr, &DirName, pData->dwObjAttrFlags, 0, NULL);
        Status = NtCreateFile(&pData->hDirectory,
                              FILE_READ_DATA,
                              &ObjAttr,
                              &IoStatus,
                              NULL,
                              FILE_ATTRIBUTE_DIRECTORY,
                              FILE_SHARE_READ | FILE_SHARE_WRITE,
                              FILE_OPEN,
                              FILE_DIRECTORY_FILE,
                              NULL,
                              0);

        if(!NT_SUCCESS(Status))
        {
            MessageBoxRc(hDlg, IDS_ERROR, IDS_E_OPENDIR, NtStatus2Text(Status), Status);
            return TRUE;
        }
    }

    // Prepare the file open
    if(NT_SUCCESS(Status))
    {
        INIT_UNICODE_STRING(FileName, pData->szFileName1);

        // If open by ID required, set the ID to the string
        if(pData->dwCreateOptions & FILE_OPEN_BY_FILE_ID)
        {
            if(ConvertStringToFileID(hDlg, IDC_FILE_NAME, FileName) != ERROR_SUCCESS)
                return TRUE;
        }

        ZeroMemory(&IoStatus, sizeof(IO_STATUS_BLOCK));
        InitializeObjectAttributes(&ObjAttr, &FileName, pData->dwObjAttrFlags, pData->hDirectory, NULL);
        Status = NtCreateFile(&pData->hFile,
                               pData->dwDesiredAccess,
                              &ObjAttr,
                              &IoStatus,
                              &pData->liAllocationSize,
                               pData->dwFileAttributes,
                               pData->dwShareAccess,
                               pData->dwCreateDisposition,
                              (pData->dwCreateOptions & 0x00FFFFFF),
                               NULL,
                               0);
        SetNtOperationResult(hDlg, Status, &IoStatus);
    }
    return TRUE;
}

static int OnNtCloseClick(HWND hDlg)
{
    TPropSheetData * pData = (TPropSheetData *)GetWindowLongPtr(hDlg, DWLP_USER);
    NTSTATUS Status = STATUS_SUCCESS;

    if(IsHandleValid(pData->hFile))
        Status = NtClose(pData->hFile);
    if(IsHandleValid(pData->hDirectory))
        NtClose(pData->hDirectory);
    pData->hDirectory = pData->hFile = NULL;

    SetNtOperationResult(hDlg, Status, NULL);
    return TRUE;
}

static int OnInitDialog(HWND hDlg, LPARAM lParam)
{
    TPropSheetData * pData;
    PROPSHEETPAGE * pPage = (PROPSHEETPAGE *)lParam;
    HWND hCombo = GetDlgItem(hDlg, IDC_CREATE_DISPOSITION);

    SetWindowLong(hDlg, DWLP_USER, (LONG)pPage->lParam);
    pData = (TPropSheetData *)pPage->lParam;
    pData->dwCreateDisposition = FILE_OPEN;

    // Initialize the combo box
    InitDialogControls(hDlg, MAKEINTRESOURCE(IDD_PAGE05_NTCREATE));
    if(hCombo != NULL)
        ComboBox_SetCurSel(hCombo, FILE_OPEN);

    return TRUE;
}

static int OnSetActive(HWND hDlg)
{
    TPropSheetData * pData = (TPropSheetData *)GetWindowLongPtr(hDlg, DWLP_USER);
    ULONG dwCreateOptions = pData->dwCreateOptions | (pData->dwCreateDisposition << 24);

    // Set directory name and file name
    SetDlgItemText(hDlg, IDC_DIRECTORY_NAME, pData->szDirName);
    ConvertToNtName(hDlg, IDC_DIRECTORY_NAME);

    SetDlgItemText(hDlg, IDC_FILE_NAME, pData->szFileName1);
    ConvertToNtName(hDlg, IDC_FILE_NAME);

    // Set the various create options
    Hex2DlgText32(hDlg, IDC_DESIRED_ACCESS, pData->dwDesiredAccess);
    Hex2DlgText64(hDlg, IDC_ALLOCATION_SIZE, pData->liAllocationSize);
    Hex2DlgText32(hDlg, IDC_FILE_ATTRIBUTES, pData->dwFileAttributes);
    Hex2DlgText32(hDlg, IDC_SHARE_ACCESS, pData->dwShareAccess);
    Hex2DlgText32(hDlg, IDC_CREATE_OPTIONS, dwCreateOptions);
    Hex2DlgText32(hDlg, IDC_OBJ_ATTR_FLAGS, pData->dwObjAttrFlags);

    SetNtOperationResult(hDlg, 0, NULL);
    return TRUE;
}

static int OnKillActive(HWND hDlg)
{
    SaveDialog(hDlg);
    return TRUE;
}

static int OnDeltaPos(HWND hDlg, NMUPDOWN * pNMUpDown)
{
    LARGE_INTEGER liAllocSize;

    liAllocSize = DlgText2Hex64(hDlg, IDC_ALLOCATION_SIZE);
    liAllocSize.QuadPart -= pNMUpDown->iDelta * 0x100;
    if(liAllocSize.QuadPart < 0)
        liAllocSize.QuadPart = 0;
    Hex2DlgText64(hDlg, IDC_ALLOCATION_SIZE, liAllocSize);
    return TRUE;
}

static int OnCommand(HWND hDlg, UINT nNotify, UINT nIDCtrl)
{
    if(nNotify == BN_CLICKED)
    {
        switch(nIDCtrl)
        {
            case IDC_BROWSE_DIR:
                return OnBrowseDirClick(hDlg);

            case IDC_BROWSE_FILE:
                return OnBrowseFileClick(hDlg);

            case IDC_BROWSE_DESIRED_ACCESS:
                return OnDesiredAccessClick(hDlg);

            case IDC_BROWSE_FILE_ATTRIBUTES:
                return OnFileAttributesClick(hDlg);
        
            case IDC_BROWSE_SHARE_ACCESS:
                return OnShareAccessClick(hDlg);

            case IDC_BROWSE_CREATE_OPTIONS:
                return OnCreateOptionsClick(hDlg);

            case IDC_BROWSE_OBJ_ATTR_FLAGS:
                return OnObjAtributesFlags(hDlg);

            case IDC_OPEN_FILE:
                return OnOpenFileClick(hDlg);

            case IDC_NTCLOSE:
                return OnNtCloseClick(hDlg);
        }
    }
    if(nNotify == EN_KILLFOCUS || nNotify == EN_SETFOCUS)
    {
        if(nIDCtrl == IDC_DIRECTORY_NAME || nIDCtrl == IDC_FILE_NAME)
        {
            ConvertToNtName(hDlg, nIDCtrl);
            return TRUE;
        }
    }

    if(nNotify == CBN_SELCHANGE)
    {
        if(nIDCtrl == IDC_CREATE_DISPOSITION)
            OnCreateDispositionChanged(hDlg);
    }

    return FALSE;
}

static int OnNotify(HWND hDlg, NMHDR * pNMHDR)
{
    switch(pNMHDR->code)
    {
        case PSN_SETACTIVE:
            return OnSetActive(hDlg);

        case PSN_KILLACTIVE:
            return OnKillActive(hDlg);

        case UDN_DELTAPOS:
            return OnDeltaPos(hDlg, (NMUPDOWN *)pNMHDR);
    }
    return FALSE;
}

//-----------------------------------------------------------------------------
// Public functions

INT_PTR CALLBACK PageProc05(HWND hDlg, UINT uMsg, WPARAM wParam, LPARAM lParam)
{
    switch(uMsg)
    {
        case WM_INITDIALOG:
            return OnInitDialog(hDlg, lParam);

        case WM_COMMAND:
            return OnCommand(hDlg, HIWORD(wParam), LOWORD(wParam));

        case WM_NOTIFY:
            return OnNotify(hDlg, (NMHDR *)lParam);
    }
    return FALSE;
}
