/*****************************************************************************/
/* Page06NtQueryInfo.cpp                  Copyright (c) Ladislav Zezula 2005 */
/*---------------------------------------------------------------------------*/
/* Description:                                                              */
/*---------------------------------------------------------------------------*/
/*   Date    Ver   Who  Comment                                              */
/* --------  ----  ---  -------                                              */
/* 08.03.05  1.00  Lad  The first version of Page06NtQueryInfo.cpp           */
/*****************************************************************************/

#include "FileTest.h"
#include "resource.h"

//-----------------------------------------------------------------------------
// Local defines

#define TYPE_BOOLEAN  0
#define TYPE_UINT8    1
#define TYPE_UINT16   2
#define TYPE_UINT32   3
#define TYPE_UINT64   4
#define TYPE_STRING   5
#define TYPE_WSTRING  6

//-----------------------------------------------------------------------------
// Helper functions

static void FillComboBox(HWND hCombo)
{
    LPCTSTR szFileInfoClass[] =
    {
        _T("FileDirectoryInformation"),
        _T("FileFullDirectoryInformation"),   
        _T("FileBothDirectoryInformation"),   
        _T("FileBasicInformation"),           
        _T("FileStandardInformation"),        
        _T("FileInternalInformation"),        
        _T("FileEaInformation"),              
        _T("FileAccessInformation"),          
        _T("FileNameInformation"),            
        _T("FileRenameInformation"),          
        _T("FileLinkInformation"),            
        _T("FileNamesInformation"),           
        _T("FileDispositionInformation"),     
        _T("FilePositionInformation"),        
        _T("FileFullEaInformation"),          
        _T("FileModeInformation"),            
        _T("FileAlignmentInformation"),       
        _T("FileAllInformation"),             
        _T("FileAllocationInformation"),      
        _T("FileEndOfFileInformation"),       
        _T("FileAlternateNameInformation"),   
        _T("FileStreamInformation"),          
        _T("FilePipeInformation"),            
        _T("FilePipeLocalInformation"),       
        _T("FilePipeRemoteInformation"),      
        _T("FileMailslotQueryInformation"),   
        _T("FileMailslotSetInformation"),     
        _T("FileCompressionInformation"),     
        _T("FileObjectIdInformation"),        
        _T("FileCompletionInformation"),      
        _T("FileMoveClusterInformation"),     
        _T("FileQuotaInformation"),           
        _T("FileReparsePointInformation"),    
        _T("FileNetworkOpenInformation"),     
        _T("FileAttributeTagInformation"),    
        _T("FileTrackingInformation"),        
        _T("FileIdBothDirectoryInformation"), 
        _T("FileIdFullDirectoryInformation"), 
        _T("FileValidDataLengthInformation"), 
        _T("FileShortNameInformation"),       
        NULL
    };

    for(int i = 0; szFileInfoClass[i] != NULL; i++)
    {
        ComboBox_AddString(hCombo, szFileInfoClass[i]);
    }

    ComboBox_SetCurSel(hCombo, (int)FileBasicInformation - 1);
}

static void UpdateDialog(HWND hDlg)
{
    TPropSheetData * pData = (TPropSheetData *)GetWindowLongPtr(hDlg, DWLP_USER);
    BOOL bEnable = FALSE;

    // Controls on the dialog will be enabled only if the
    // handle is valid and it is a NT handle

    if(IsHandleValid(pData->hFile))
        bEnable = TRUE;

    EnableDlgItems(hDlg, bEnable, IDC_QUERYINFO, 0);
}

//-----------------------------------------------------------------------------
// Filling the tree view

static HTREEITEM InsertRootItem(HWND hTreeView, HTREEITEM hItem, LPCTSTR szStructName, PVOID pParam)
{
    TVINSERTSTRUCT tvis;

    tvis.hParent      = hItem;
    tvis.hInsertAfter = TVI_LAST;
    tvis.item.mask    = TVIF_TEXT | TVIF_PARAM;
    tvis.item.pszText = (LPTSTR)szStructName;
    tvis.item.lParam  = (LPARAM)pParam;
    return TreeView_InsertItem(hTreeView, &tvis);
}

static HTREEITEM InsertStructMember(HWND hTreeView, HTREEITEM hParentItem, LPCTSTR szName, PBYTE pbData, ULONG & dwOffset, int nType, ULONG ByteSize = 0)
{
    TVINSERTSTRUCT tvis;
    TCHAR szItemText[512] = _T("");
    LPTSTR szTemp = szItemText;

    // Prepare the item text
    szTemp += _stprintf(szItemText, _T("%s: "), szName);
    pbData += dwOffset;

    switch(nType)
    {
        case TYPE_BOOLEAN:
        {
            PBOOLEAN pBoolean = (PBOOLEAN)pbData;

            _tcscpy(szTemp, *pBoolean ? _T("TRUE") : _T("FALSE"));
            dwOffset += sizeof(BOOLEAN);
            break;
        }

        case TYPE_UINT8:
        {
            PUCHAR pUchar = (PUCHAR)pbData;

            _stprintf(szTemp, _T("%02lX"), *pUchar);
            dwOffset += sizeof(UCHAR);
            break;
        }

        case TYPE_UINT16:
        {
            PUSHORT pUshort = (PUSHORT)pbData;

            _stprintf(szTemp, _T("%04lX"), *pUshort);
            dwOffset += sizeof(USHORT);
            break;
        }

        case TYPE_UINT32:
        {
            PULONG pUlong = (PULONG)pbData;

            _stprintf(szTemp, _T("%08lX"), *pUlong);
            dwOffset += sizeof(ULONG);
            break;
        }

        case TYPE_UINT64:
        {
            PLONGLONG pLongLong = (PLONGLONG)pbData;

            _stprintf(szTemp, _T("%016I64X"), *pLongLong);
            dwOffset += sizeof(LONGLONG);
            break;
        }

        case TYPE_STRING:
        {
            MultiByteToWideChar(CP_ACP, MB_PRECOMPOSED, (PCHAR)pbData, ByteSize, szTemp, ByteSize);
            szTemp[(ByteSize / sizeof(WCHAR))] = 0;
            dwOffset += ByteSize;
            break;
        }

        case TYPE_WSTRING:
        {
            wcsncpy(szTemp, (PWCHAR)pbData, (ByteSize / sizeof(WCHAR)));
            szTemp[(ByteSize / sizeof(WCHAR))] = 0;
            dwOffset += ByteSize;
            break;
        }
    }

    // Insert the item
    tvis.hParent      = hParentItem;
    tvis.hInsertAfter = TVI_LAST;
    tvis.item.mask    = TVIF_TEXT;
    tvis.item.pszText = szItemText;
    return TreeView_InsertItem(hTreeView, &tvis);
}

static void InsertFileBasicInfo(HWND hTreeView, HTREEITEM hItem, PBYTE pbFileInfo)
{
    ULONG dwOffset = 0;

    hItem = InsertRootItem(hTreeView, hItem, _T("FILE_BASIC_INFORMATION"), (hItem == TVI_ROOT) ? pbFileInfo : NULL);
    InsertStructMember(hTreeView, hItem, _T("CreationTime"), pbFileInfo, dwOffset, TYPE_UINT64);
    InsertStructMember(hTreeView, hItem, _T("LastAccessTime"), pbFileInfo, dwOffset, TYPE_UINT64);
    InsertStructMember(hTreeView, hItem, _T("LastWriteTime"), pbFileInfo, dwOffset, TYPE_UINT64);
    InsertStructMember(hTreeView, hItem, _T("ChangeTime"), pbFileInfo, dwOffset, TYPE_UINT64);
    InsertStructMember(hTreeView, hItem, _T("FileAttributes"), pbFileInfo, dwOffset, TYPE_UINT32);
}

static void InsertFileStandardInfo(HWND hTreeView, HTREEITEM hItem, PBYTE pbFileInfo)
{
    ULONG dwOffset = 0;

    hItem = InsertRootItem(hTreeView, hItem, _T("FILE_STANDARD_INFORMATION"), (hItem == TVI_ROOT) ? pbFileInfo : NULL);
    InsertStructMember(hTreeView, hItem, _T("AllocationSize"), pbFileInfo, dwOffset, TYPE_UINT64);
    InsertStructMember(hTreeView, hItem, _T("EndOfFile"), pbFileInfo, dwOffset, TYPE_UINT64);
    InsertStructMember(hTreeView, hItem, _T("NumberOfLinks"), pbFileInfo, dwOffset, TYPE_UINT32);
    InsertStructMember(hTreeView, hItem, _T("DeletePending"), pbFileInfo, dwOffset, TYPE_BOOLEAN);
    InsertStructMember(hTreeView, hItem, _T("Directory"), pbFileInfo, dwOffset, TYPE_BOOLEAN);
}

static void InsertFileInternalInfo(HWND hTreeView, HTREEITEM hItem, PBYTE pbFileInfo)
{
    ULONG dwOffset = 0;

    hItem = InsertRootItem(hTreeView, hItem, _T("FILE_INTERNAL_INFORMATION"), (hItem == TVI_ROOT) ? pbFileInfo : NULL);
    InsertStructMember(hTreeView, hItem, _T("IndexNumber"), pbFileInfo, dwOffset, TYPE_UINT64);
}

static void InsertFileEaInfo(HWND hTreeView, HTREEITEM hItem, PBYTE pbFileInfo)
{
    ULONG dwOffset = 0;

    hItem = InsertRootItem(hTreeView, hItem, _T("FILE_EA_INFORMATION"), (hItem == TVI_ROOT) ? pbFileInfo : NULL);
    InsertStructMember(hTreeView, hItem, _T("EaSize"), pbFileInfo, dwOffset, TYPE_UINT32);
}

static void InsertFileNameInfo(HWND hTreeView, HTREEITEM hItem, PBYTE pbFileInfo)
{
    PFILE_NAME_INFORMATION FileNameInfo = (PFILE_NAME_INFORMATION)pbFileInfo;
    ULONG dwOffset = 0;

    hItem = InsertRootItem(hTreeView, hItem, _T("FILE_NAME_INFORMATION"), (hItem == TVI_ROOT) ? pbFileInfo : NULL);
    InsertStructMember(hTreeView, hItem, _T("FileNameLength"), pbFileInfo, dwOffset, TYPE_UINT32);
    InsertStructMember(hTreeView, hItem, _T("FileName"), pbFileInfo, dwOffset, TYPE_WSTRING, FileNameInfo->FileNameLength);
}

static void InsertFileAccessInfo(HWND hTreeView, HTREEITEM hItem, PBYTE pbFileInfo)
{
    ULONG dwOffset = 0;

    hItem = InsertRootItem(hTreeView, hItem, _T("FILE_ACCESS_INFORMATION"), (hItem == TVI_ROOT) ? pbFileInfo : NULL);
    InsertStructMember(hTreeView, hItem, _T("AccessFlags"), pbFileInfo, dwOffset, TYPE_UINT64);
}

static void InsertFilePositionInfo(HWND hTreeView, HTREEITEM hItem, PBYTE pbFileInfo)
{
    ULONG dwOffset = 0;

    hItem = InsertRootItem(hTreeView, hItem, _T("FILE_POSITION_INFORMATION"), (hItem == TVI_ROOT) ? pbFileInfo : NULL);
    InsertStructMember(hTreeView, hItem, _T("CurrentByteOffset"), pbFileInfo, dwOffset, TYPE_UINT64);
}

static void InsertFileModeInfo(HWND hTreeView, HTREEITEM hItem, PBYTE pbFileInfo)
{
    ULONG dwOffset = 0;

    hItem = InsertRootItem(hTreeView, hItem, _T("FILE_MODE_INFORMATION"), (hItem == TVI_ROOT) ? pbFileInfo : NULL);
    InsertStructMember(hTreeView, hItem, _T("Mode"), pbFileInfo, dwOffset, TYPE_UINT32);
}

static void InsertFileAlignmentInfo(HWND hTreeView, HTREEITEM hItem, PBYTE pbFileInfo)
{
    ULONG dwOffset = 0;

    hItem = InsertRootItem(hTreeView, hItem, _T("FILE_ALIGNMENT_INFORMATION"), (hItem == TVI_ROOT) ? pbFileInfo : NULL);
    InsertStructMember(hTreeView, hItem, _T("AlignmentRequirement"), pbFileInfo, dwOffset, TYPE_UINT32);
}

static void InsertFileStreamInfo(HWND hTreeView, HTREEITEM hRoot, PBYTE pbFileInfo)
{
    PFILE_STREAM_INFORMATION FileStreamInfo;
    HTREEITEM hItem;
    ULONG dwOffset;
    TCHAR szName[128];
    int nIndex = 0;

    // Insert infos about the streams
    do
    {
        FileStreamInfo = (PFILE_STREAM_INFORMATION)pbFileInfo;
        dwOffset = 0;

        _stprintf(szName, _T("[%u]"), nIndex++);
        hItem = InsertRootItem(hTreeView, hRoot, szName, NULL);

        InsertStructMember(hTreeView, hItem, _T("NextEntryOffset"), pbFileInfo, dwOffset, TYPE_UINT32);
        InsertStructMember(hTreeView, hItem, _T("StreamNameLength"), pbFileInfo, dwOffset, TYPE_UINT32);
        InsertStructMember(hTreeView, hItem, _T("StreamSize"), pbFileInfo, dwOffset, TYPE_UINT64);
        InsertStructMember(hTreeView, hItem, _T("StreamAllocationSize"), pbFileInfo, dwOffset, TYPE_UINT64);
        InsertStructMember(hTreeView, hItem, _T("StreamName"), pbFileInfo, dwOffset, TYPE_WSTRING, FileStreamInfo->StreamNameLength);

        // Move to the next stream
        pbFileInfo += FileStreamInfo->NextEntryOffset;
    }
    while(FileStreamInfo->NextEntryOffset != 0);
}


static void FillTreeViewWithData(HWND hTreeView, PVOID pFileInfo, FILE_INFORMATION_CLASS FileInfoClass)
{
    HTREEITEM hRoot = NULL;
    PBYTE pbFileInfo = (PBYTE)pFileInfo;
    ULONG dwOffset = 0;

    switch(FileInfoClass)
    {
        case FileBasicInformation:
            InsertFileBasicInfo(hTreeView, TVI_ROOT, pbFileInfo);
            break;

        case FileStandardInformation:
            InsertFileStandardInfo(hTreeView, TVI_ROOT, pbFileInfo);
            break;

        case FileInternalInformation:
            InsertFileInternalInfo(hTreeView, TVI_ROOT, pbFileInfo);
            break;

        case FileEaInformation:
            InsertFileEaInfo(hTreeView, TVI_ROOT, pbFileInfo);
            break;

        case FileAccessInformation:
            InsertFileAccessInfo(hTreeView, TVI_ROOT, pbFileInfo);
            break;

        case FileNameInformation:
        case FileAlternateNameInformation:
        case FileShortNameInformation:
            InsertFileNameInfo(hTreeView, TVI_ROOT, pbFileInfo);
            break;

        case FilePositionInformation:
            InsertFilePositionInfo(hTreeView, TVI_ROOT, pbFileInfo);
            break;

        case FileModeInformation:
            InsertFileModeInfo(hTreeView, TVI_ROOT, pbFileInfo);
            break;

        case FileAlignmentInformation:
            InsertFileAlignmentInfo(hTreeView, TVI_ROOT, pbFileInfo);
            break;

        case FileAllInformation:
        {
            PFILE_ALL_INFORMATION FileAllInfo = (PFILE_ALL_INFORMATION)pFileInfo;

            hRoot = InsertRootItem(hTreeView, TVI_ROOT, _T("FILE_ALL_INFORMATION"), pFileInfo);
            InsertFileBasicInfo(hTreeView, hRoot, (PBYTE)&FileAllInfo->BasicInformation);
            InsertFileStandardInfo(hTreeView, hRoot, (PBYTE)&FileAllInfo->StandardInformation);
            InsertFileInternalInfo(hTreeView, hRoot, (PBYTE)&FileAllInfo->InternalInformation);
            InsertFileEaInfo(hTreeView, hRoot, (PBYTE)&FileAllInfo->EaInformation);
            InsertFileAccessInfo(hTreeView, hRoot, (PBYTE)&FileAllInfo->AccessInformation);
            InsertFilePositionInfo(hTreeView, hRoot, (PBYTE)&FileAllInfo->PositionInformation);
            InsertFileModeInfo(hTreeView, hRoot, (PBYTE)&FileAllInfo->ModeInformation);
            InsertFileAlignmentInfo(hTreeView, hRoot, (PBYTE)&FileAllInfo->AlignmentInformation);
            InsertFileNameInfo(hTreeView, hRoot, (PBYTE)&FileAllInfo->NameInformation);
            break;
        }

        case FileAllocationInformation:
            hRoot = InsertRootItem(hTreeView, TVI_ROOT, _T("FILE_ALLOCATION_INFORMATION"), pFileInfo);
            InsertStructMember(hTreeView, hRoot, _T("AllocationSize"), pbFileInfo, dwOffset, TYPE_UINT64);
            break;

        case FileEndOfFileInformation:
            hRoot = InsertRootItem(hTreeView, TVI_ROOT, _T("FILE_END_OF_FILE_INFORMATION"), pFileInfo);
            InsertStructMember(hTreeView, hRoot, _T("EndOfFile"), pbFileInfo, dwOffset, TYPE_UINT64);
            break;

        case FileStreamInformation:
            hRoot = InsertRootItem(hTreeView, hRoot, _T("FILE_STREAM_INFORMATION"), pbFileInfo);
            InsertFileStreamInfo(hTreeView, hRoot, pbFileInfo);
            break;

        case FilePipeInformation:
            hRoot = InsertRootItem(hTreeView, hRoot, _T("FILE_PIPE_INFORMATION"), pbFileInfo);
            InsertStructMember(hTreeView, hRoot, _T("ReadMode"), pbFileInfo, dwOffset, TYPE_UINT32);
            InsertStructMember(hTreeView, hRoot, _T("CompletionMode"), pbFileInfo, dwOffset, TYPE_UINT32);
            break;

        case FilePipeLocalInformation:
            hRoot = InsertRootItem(hTreeView, hRoot, _T("FILE_PIPE_LOCAL_INFORMATION"), pbFileInfo);
            InsertStructMember(hTreeView, hRoot, _T("NamedPipeType"), pbFileInfo, dwOffset, TYPE_UINT32);
            InsertStructMember(hTreeView, hRoot, _T("NamedPipeConfiguration"), pbFileInfo, dwOffset, TYPE_UINT32);
            InsertStructMember(hTreeView, hRoot, _T("MaximumInstances"), pbFileInfo, dwOffset, TYPE_UINT32);
            InsertStructMember(hTreeView, hRoot, _T("CurrentInstances"), pbFileInfo, dwOffset, TYPE_UINT32);
            InsertStructMember(hTreeView, hRoot, _T("InboundQuota"), pbFileInfo, dwOffset, TYPE_UINT32);
            InsertStructMember(hTreeView, hRoot, _T("ReadDataAvailable"), pbFileInfo, dwOffset, TYPE_UINT32);
            InsertStructMember(hTreeView, hRoot, _T("OutboundQuota"), pbFileInfo, dwOffset, TYPE_UINT32);
            InsertStructMember(hTreeView, hRoot, _T("WriteQuotaAvailable"), pbFileInfo, dwOffset, TYPE_UINT32);
            InsertStructMember(hTreeView, hRoot, _T("NamedPipeState"), pbFileInfo, dwOffset, TYPE_UINT32);
            InsertStructMember(hTreeView, hRoot, _T("NamedPipeEnd"), pbFileInfo, dwOffset, TYPE_UINT32);
            break;

        case FilePipeRemoteInformation:
            hRoot = InsertRootItem(hTreeView, hRoot, _T("FILE_PIPE_REMOTE_INFORMATION"), pbFileInfo);
            InsertStructMember(hTreeView, hRoot, _T("CollectDataTime"), pbFileInfo, dwOffset, TYPE_UINT64);
            InsertStructMember(hTreeView, hRoot, _T("MaximumCollectionCount"), pbFileInfo, dwOffset, TYPE_UINT32);
            break;

        case FileMailslotQueryInformation:
            hRoot = InsertRootItem(hTreeView, hRoot, _T("FILE_MAILSLOT_QUERY_INFORMATION"), pbFileInfo);
            InsertStructMember(hTreeView, hRoot, _T("ReadTimeout"), pbFileInfo, dwOffset, TYPE_UINT64);
            break;

        case FileCompressionInformation:
            hRoot = InsertRootItem(hTreeView, hRoot, _T("FILE_COMPRESSION_INFORMATION"), pbFileInfo);
            InsertStructMember(hTreeView, hRoot, _T("CompressedFileSize"), pbFileInfo, dwOffset, TYPE_UINT64);
            InsertStructMember(hTreeView, hRoot, _T("CompressionFormat"), pbFileInfo, dwOffset, TYPE_UINT16);
            InsertStructMember(hTreeView, hRoot, _T("CompressionUnitShift"), pbFileInfo, dwOffset, TYPE_UINT8);
            InsertStructMember(hTreeView, hRoot, _T("ChunkShift"), pbFileInfo, dwOffset, TYPE_UINT8);
            InsertStructMember(hTreeView, hRoot, _T("ClusterShift"), pbFileInfo, dwOffset, TYPE_UINT8);
            InsertStructMember(hTreeView, hRoot, _T("Reserved[0]"), pbFileInfo, dwOffset, TYPE_UINT8);
            InsertStructMember(hTreeView, hRoot, _T("Reserved[1]"), pbFileInfo, dwOffset, TYPE_UINT8);
            InsertStructMember(hTreeView, hRoot, _T("Reserved[2]"), pbFileInfo, dwOffset, TYPE_UINT8);
            break;

        case FileObjectIdInformation:
            hRoot = InsertRootItem(hTreeView, hRoot, _T("FILE_OBJECTID_INFORMATION"), pbFileInfo);
            InsertStructMember(hTreeView, hRoot, _T("FileReference"), pbFileInfo, dwOffset, TYPE_UINT64);
            InsertStructMember(hTreeView, hRoot, _T("ObjectId"), pbFileInfo, dwOffset, TYPE_STRING, 16);
            InsertStructMember(hTreeView, hRoot, _T("BirthVolumeId"), pbFileInfo, dwOffset, TYPE_STRING, 16);
            InsertStructMember(hTreeView, hRoot, _T("BirthObjectId"), pbFileInfo, dwOffset, TYPE_STRING, 16);
            InsertStructMember(hTreeView, hRoot, _T("DomainId"), pbFileInfo, dwOffset, TYPE_STRING, 16);
            break;

        case FileNetworkOpenInformation:
            hRoot = InsertRootItem(hTreeView, hRoot, _T("FILE_NETWORK_OPEN_INFORMATION"), pbFileInfo);
            InsertStructMember(hTreeView, hRoot, _T("CreationTime"), pbFileInfo, dwOffset, TYPE_UINT64);
            InsertStructMember(hTreeView, hRoot, _T("LastAccessTime"), pbFileInfo, dwOffset, TYPE_UINT64);
            InsertStructMember(hTreeView, hRoot, _T("LastWriteTime"), pbFileInfo, dwOffset, TYPE_UINT64);
            InsertStructMember(hTreeView, hRoot, _T("ChangeTime"), pbFileInfo, dwOffset, TYPE_UINT64);
            InsertStructMember(hTreeView, hRoot, _T("AllocationSize"), pbFileInfo, dwOffset, TYPE_UINT64);
            InsertStructMember(hTreeView, hRoot, _T("EndOfFile"), pbFileInfo, dwOffset, TYPE_UINT64);
            InsertStructMember(hTreeView, hRoot, _T("FileAttributes"), pbFileInfo, dwOffset, TYPE_UINT32);
            break;

        case FileAttributeTagInformation:
            hRoot = InsertRootItem(hTreeView, hRoot, _T("FILE_ATTRIBUTE_TAG_INFORMATION"), pbFileInfo);
            InsertStructMember(hTreeView, hRoot, _T("FileAttributes"), pbFileInfo, dwOffset, TYPE_UINT32);
            InsertStructMember(hTreeView, hRoot, _T("ReparseTag"), pbFileInfo, dwOffset, TYPE_UINT32);
            break;

        case FileValidDataLengthInformation:
            hRoot = InsertRootItem(hTreeView, hRoot, _T("FILE_VALID_DATA_LENGTH_INFORMATION"), pbFileInfo);
            InsertStructMember(hTreeView, hRoot, _T("ValidDataLength"), pbFileInfo, dwOffset, TYPE_UINT64);
            break;

        default:
            InsertRootItem(hTreeView, TVI_ROOT, _T("STRUCTURE_NOT_SUPPORTED"), pFileInfo);
            break;
    }

    // If a root item, expand it.
    TreeView_Expand(hTreeView, hRoot, TVE_EXPAND);
}

//-----------------------------------------------------------------------------
// Message handlers

static int OnInitDialog(HWND hDlg, LPARAM lParam)
{
    PROPSHEETPAGE * pPage = (PROPSHEETPAGE *)lParam;

    SetWindowLong(hDlg, DWLP_USER, (LONG)pPage->lParam);
    FillComboBox(GetDlgItem(hDlg, IDC_FILE_INFO_CLASS));
    UpdateDialog(hDlg);
    return TRUE;
}

static int OnSetActive(HWND hDlg)
{
    UpdateDialog(hDlg);
    return TRUE;
}

static int OnKillActive(HWND /* hDlg */)
{
    return TRUE;
}

static int OnDeleteItem(HWND /* hDlg */, NMTREEVIEW * pNMTreeView)
{
    if(pNMTreeView->itemOld.lParam != 0)
    {
        HeapFree(g_hHeap, 0, (PVOID)pNMTreeView->itemOld.lParam);
    }
    return TRUE;
}


static int OnQueryInfoClick(HWND hDlg)
{
    FILE_INFORMATION_CLASS FileInfoClass;
    TPropSheetData * pData = (TPropSheetData *)GetWindowLongPtr(hDlg, DWLP_USER);
    IO_STATUS_BLOCK IoStatus = {0};
    NTSTATUS Status = STATUS_SUCCESS;
    PVOID pFileInfo = NULL;
    ULONG Length = 4096;                    // Hopefully it will be enough
    HWND hTreeView = GetDlgItem(hDlg, IDC_FILE_INFO);
    HWND hCombo = GetDlgItem(hDlg, IDC_FILE_INFO_CLASS);

    // Test if the handle is an NT file handle and get the file info class
    if(IsHandleValid(pData->hFile) == FALSE)
        return FALSE;
    FileInfoClass = (FILE_INFORMATION_CLASS)(ComboBox_GetCurSel(hCombo) + 1);

    // Allocate buffer
    if(NT_SUCCESS(Status))
    {
        pFileInfo = HeapAlloc(g_hHeap, HEAP_ZERO_MEMORY, Length);
        if(pFileInfo == NULL)
            Status = STATUS_NO_MEMORY;
    }

    // Call NtQueryInformationFile
    if(NT_SUCCESS(Status))
    {
        Status = NtQueryInformationFile(pData->hFile,
                                       &IoStatus,
                                        pFileInfo,
                                        Length,
                                        FileInfoClass);
    }                                        

    // Set the status and the returned length
    TreeView_DeleteAllItems(hTreeView);
    SetDlgItemText(hDlg, IDC_RESULT_STATUS, NtStatus2Text(Status));
    SetDlgItemInt(hDlg, IDC_RESULT_LENGTH, (UINT)IoStatus.Information, FALSE);
    
    // If success, fill the tree view with the structure
    if(NT_SUCCESS(Status))
    {
        FillTreeViewWithData(hTreeView, pFileInfo, FileInfoClass);
        pFileInfo = NULL;
    }

    // Cleanup & return
    if(pFileInfo != NULL)
        HeapFree(g_hHeap, 0, pFileInfo);
    return TRUE;
}

static int OnCommand(HWND hDlg, UINT nNotify, UINT nIDCtrl)
{
    if(nNotify == BN_CLICKED)
    {
        switch(nIDCtrl)
        {
            case IDC_QUERYINFO:
                OnQueryInfoClick(hDlg);
                return TRUE;
        }
    }
    return FALSE;
}

static int OnNotify(HWND hDlg, NMHDR * pNMHDR)
{
    switch(pNMHDR->code)
    {
        case PSN_SETACTIVE:
            return OnSetActive(hDlg);

        case PSN_KILLACTIVE:
            return OnKillActive(hDlg);

        case TVN_DELETEITEM:
            return OnDeleteItem(hDlg, (LPNMTREEVIEW)pNMHDR);
    }
    return FALSE;
}

//-----------------------------------------------------------------------------
// Public functions

INT_PTR CALLBACK PageProc06(HWND hDlg, UINT uMsg, WPARAM wParam, LPARAM lParam)
{
    switch(uMsg)
    {
        case WM_INITDIALOG:
            return OnInitDialog(hDlg, lParam);

        case WM_COMMAND:
            return OnCommand(hDlg, HIWORD(wParam), LOWORD(wParam));

        case WM_NOTIFY:
            return OnNotify(hDlg, (NMHDR *)lParam);
    }
    return FALSE;
}
