/*****************************************************************************/
/* WinMain.cpp                            Copyright (c) Ladislav Zezula 2003 */
/*---------------------------------------------------------------------------*/
/* A file that simulates access on a file                                    */
/*---------------------------------------------------------------------------*/
/*   Date    Ver   Who  Comment                                              */
/* --------  ----  ---  -------                                              */
/* 14.07.03  1.00  Lad  The first version of WinMain.cpp                     */
/*****************************************************************************/

#include "FileTest.h"
#include "resource.h"

#pragma comment(lib, "Comctl32.lib")

//-----------------------------------------------------------------------------
// Global variables

HINSTANCE g_hInst;
HANDLE g_hHeap;

//-----------------------------------------------------------------------------
// Main dialog

INT_PTR FileTestDialog(HWND hParent, TPropSheetData * pData)
{
    PROPSHEETHEADER psh;
    PROPSHEETPAGE psp[6];
    INT_PTR nResult;
    TCHAR szUserName[256] = _T("");
    TCHAR szTitle[256] = _T("");
    TCHAR szFmt[256] = _T("");
    int nPages = 0;

    // Construct the dialog title
    if(LoadString(g_hInst, IDS_APP_TITLE, szFmt, _tsize(szFmt)))
    {
        DWORD dwSize = _tsize(szUserName);

        GetUserName(szUserName, &dwSize);
        _stprintf(szTitle, szFmt, szUserName);
    }

    // Fill the property sheet header
    ZeroMemory(&psh, sizeof(PROPSHEETHEADER));
    psh.dwSize     = sizeof(PROPSHEETHEADER);
    psh.dwFlags    = PSH_PROPSHEETPAGE | PSH_USEICONID | PSH_NOAPPLYNOW | PSH_NOCONTEXTHELP;
    psh.hwndParent = hParent;
    psh.hInstance  = g_hInst;
    psh.pszIcon    = MAKEINTRESOURCE(IDI_FILE_TEST);
    psh.pszCaption = szTitle;
    psh.nStartPage = 0;
    psh.ppsp       = psp;

    // If an NT-name entered, choose the second page
    if(!_tcsncmp(pData->szFileName1, _T("\\??\\"), 4))
        psh.nStartPage = 1;

    // Fill the first page
    ZeroMemory(&psp[nPages], sizeof(PROPSHEETPAGE));
    psp[nPages].dwSize      = sizeof(PROPSHEETPAGE);
    psp[nPages].dwFlags     = PSP_DEFAULT;
    psp[nPages].hInstance   = g_hInst;
    psp[nPages].pszTemplate = MAKEINTRESOURCE(IDD_PAGE01_CREATE);
    psp[nPages].pfnDlgProc  = PageProc01;
    psp[nPages].lParam      = (LPARAM)pData;
    nPages++;

    // Fill the "ReadFile"
    ZeroMemory(&psp[nPages], sizeof(PROPSHEETPAGE));
    psp[nPages].dwSize      = sizeof(PROPSHEETPAGE);
    psp[nPages].dwFlags     = PSP_DEFAULT;
    psp[nPages].hInstance   = g_hInst;
    psp[nPages].pszTemplate = MAKEINTRESOURCE(IDD_PAGE02_READFILE);
    psp[nPages].pfnDlgProc  = PageProc02;
    psp[nPages].lParam      = (LPARAM)pData;
    nPages++;

    // Fill the "WriteFile". It uses the same handler like "ReadFile"
    ZeroMemory(&psp[nPages], sizeof(PROPSHEETPAGE));
    psp[nPages].dwSize      = sizeof(PROPSHEETPAGE);
    psp[nPages].dwFlags     = PSP_DEFAULT;
    psp[nPages].hInstance   = g_hInst;
    psp[nPages].pszTemplate = MAKEINTRESOURCE(IDD_PAGE03_WRITEFILE);
    psp[nPages].pfnDlgProc  = PageProc02;
    psp[nPages].lParam      = (LPARAM)pData;
    nPages++;

    // Fill the "File Ops".
    ZeroMemory(&psp[nPages], sizeof(PROPSHEETPAGE));
    psp[nPages].dwSize      = sizeof(PROPSHEETPAGE);
    psp[nPages].dwFlags     = PSP_DEFAULT;
    psp[nPages].hInstance   = g_hInst;
    psp[nPages].pszTemplate = MAKEINTRESOURCE(IDD_PAGE04_FILEOPS);
    psp[nPages].pfnDlgProc  = PageProc04;
    psp[nPages].lParam      = (LPARAM)pData;
    nPages++;

    // Fill the "NtCreateFile" page
    ZeroMemory(&psp[nPages], sizeof(PROPSHEETPAGE));
    psp[nPages].dwSize      = sizeof(PROPSHEETPAGE);
    psp[nPages].dwFlags     = PSP_DEFAULT;
    psp[nPages].hInstance   = g_hInst;
    psp[nPages].pszTemplate = MAKEINTRESOURCE(IDD_PAGE05_NTCREATE);
    psp[nPages].pfnDlgProc  = PageProc05;
    psp[nPages].lParam      = (LPARAM)pData;
    nPages++;

    // Fill the "NT File Ops".
    ZeroMemory(&psp[nPages], sizeof(PROPSHEETPAGE));
    psp[nPages].dwSize      = sizeof(PROPSHEETPAGE);
    psp[nPages].dwFlags     = PSP_DEFAULT;
    psp[nPages].hInstance   = g_hInst;
    psp[nPages].pszTemplate = MAKEINTRESOURCE(IDD_PAGE06_NTFILEOPS);
    psp[nPages].pfnDlgProc  = PageProc06;
    psp[nPages].lParam      = (LPARAM)pData;
    nPages++;

    // Run the property sheet dialog
    psh.nPages = nPages;
    nResult = PropertySheet(&psh);
    return nResult;
}

//-----------------------------------------------------------------------------
// WinMain

int WINAPI _tWinMain(HINSTANCE hInstance, HINSTANCE, LPSTR, int)
{
    TPropSheetData * pData =  new TPropSheetData;

    // Save the instance
    g_hInst = hInstance;
    g_hHeap = GetProcessHeap();

    InitCommonControls();

    // Fill the dialog data
    ZeroMemory(pData, sizeof(TPropSheetData));
    if(__argc > 1)
        _tcscpy(pData->szDirName, __targv[1]);
    if(__argc > 2)
        _tcscpy(pData->szFileName1, __targv[2]);
    if(__argc > 3)
        _tcscpy(pData->szFileName2, __targv[3]);
    pData->dwDesiredAccess     = GENERIC_READ;
    pData->dwFileAttributes    = FILE_ATTRIBUTE_NORMAL;
    pData->dwShareAccess       = FILE_SHARE_READ;
    pData->dwCreateOptions     = FILE_NON_DIRECTORY_FILE;
    pData->dwObjAttrFlags      = OBJ_CASE_INSENSITIVE;

    // Call the dialog
    FileTestDialog(NULL, pData);
    delete pData;
    return 0;
}
