/*****************************************************************************/
/* BrowseForDirectory.cpp                 Copyright (c) Ladislav Zezula 2004 */
/*---------------------------------------------------------------------------*/
/* Description:                                                              */
/*---------------------------------------------------------------------------*/
/*   Date    Ver   Who  Comment                                              */
/* --------  ----  ---  -------                                              */
/* 29.05.04  1.00  Lad  The first version of BrowseForDirectory.cpp          */
/*****************************************************************************/

#define __UTILS_SELF
#include "Utils.h"
#include <shlobj.h>

//-----------------------------------------------------------------------------
// Local functions

static WNDPROC OldBffDialogProc = NULL;

static LRESULT CALLBACK BFFDialogProc2(HWND hwnd, UINT uMsg, WPARAM wParam, LPARAM lParam)
{
    if(uMsg == WM_NOTIFY)
    {
        HTREEITEM hItem = NULL;
        TVITEM tvi;
        NMHDR * pNMHDR = (NMHDR *)lParam;

        if(pNMHDR->code == NM_DBLCLK)
        {
            // If the node has no more subnodes,
            // generate the "OK" button click
            hItem = TreeView_GetSelection(pNMHDR->hwndFrom);

            if(hItem != NULL)
            {
                ZeroMemory(&tvi, sizeof(TVITEM));
                tvi.mask  = TVIF_CHILDREN;
                tvi.hItem = hItem;
                TreeView_GetItem(pNMHDR->hwndFrom, &tvi);

                if(tvi.cChildren == 0)
                    PostMessage(hwnd, WM_COMMAND, MAKELONG(IDOK, BN_CLICKED), 0);
            }
        }
    }
    return CallWindowProc(OldBffDialogProc, hwnd, uMsg, wParam, lParam);
}

static int CALLBACK BFFDialogProc(HWND hwnd, UINT uMsg, LPARAM /* lParam */, LPARAM lpData)
{
    if(uMsg == BFFM_INITIALIZED)
    {
        // Set the selection
        SendMessage(hwnd, BFFM_SETSELECTION, TRUE, lpData);

        // Subclass the dialog so we can receive WM_NOTIFY
        OldBffDialogProc = (WNDPROC)GetWindowLongPtr(hwnd, GWLP_WNDPROC);
        SetWindowLongPtr(hwnd, GWLP_WNDPROC, (LONG_PTR)BFFDialogProc2);
    }
    return 0;
}

//-----------------------------------------------------------------------------
// Public functions

int BrowseForDirectory(HWND hDlg, UINT nIDEdit, UINT nIDTitle)
{
    LPITEMIDLIST lpidl;
    BROWSEINFO   bi;
    TCHAR szFileName[MAX_PATH] = _T("");
    TCHAR szTitle[256] = _T("");
    BOOL bResult = FALSE;

    GetDlgItemText(hDlg, nIDEdit, szFileName, _tsize(szFileName));
    LoadString(g_hInst, nIDTitle, szTitle, _tsize(szTitle));

    ZeroMemory(&bi, sizeof(BROWSEINFO));
    bi.hwndOwner      = hDlg;           // Set the dialog parent
    bi.pszDisplayName = szFileName;     // Set folder to be browsed
    bi.lpszTitle      = szTitle;        // Dialog title
    bi.ulFlags        = BIF_RETURNONLYFSDIRS;
    bi.lParam         = (LPARAM)szFileName;
    bi.lpfn           = BFFDialogProc;

    if((lpidl = SHBrowseForFolder(&bi)) != NULL)
    {
        IMalloc * pMalloc = NULL;

        SHGetPathFromIDList(lpidl, szFileName);
        SetDlgItemText(hDlg, nIDEdit, szFileName);

        SHGetMalloc(&pMalloc);
        pMalloc->Free(lpidl);
        pMalloc->Release();
        bResult = TRUE;
    }

    // Release the IMalloc interface
    return bResult;
}
