/*****************************************************************************/
/* BrowseForFile.cpp                      Copyright (c) Ladislav Zezula 2004 */
/*---------------------------------------------------------------------------*/
/* Description:                                                              */
/*---------------------------------------------------------------------------*/
/*   Date    Ver   Who  Comment                                              */
/* --------  ----  ---  -------                                              */
/* 07.07.04  1.00  Lad  The first version of BrowseForFile.cpp               */
/*****************************************************************************/

#define __UTILS_SELF
#include "Utils.h"

// Browses for a file. The initial dir and/or the file may be
// entered by a string or by a control ID.
int BrowseForFile(HWND hParent, LPTSTR szDir, LPTSTR szFile, UINT nIDTitle, UINT nIDFilters)
{
    OPENFILENAME ofn;
    LPTSTR szPlainName = NULL;
    TCHAR szInitialDir[MAX_PATH] = _T("");
    TCHAR szFileName[MAX_PATH] = _T("");
    TCHAR szFilters[256] = _T("");
    TCHAR szTitle[256] = _T("");
    BOOL bResult = FALSE;

    // Load the title
    LoadString(g_hInst, nIDTitle, szTitle, _tsize(szTitle));

    // Resolve the initial dir
    if(szDir != NULL)
    {
        if((DWORD_PTR)szDir <= 0x10000)
            GetDlgItemText(hParent, (UINT)(UINT_PTR)szDir, szInitialDir, _tsize(szInitialDir));
        else
            _tcscpy(szInitialDir, szDir);

        // If an NT native name, clear it
        if(!_tcsncmp(szInitialDir, _T("\\??\\"), 4))
            memmove(szInitialDir, &szInitialDir[4], (sizeof(szInitialDir) - 4 * sizeof(TCHAR)));
    }

    // Resolve the file name
    if(szFile != NULL)
    {
        if((DWORD_PTR)szFile <= 0x10000)
            GetDlgItemText(hParent, (UINT)(UINT_PTR)szFile, szFileName, _tsize(szFileName));
        else
            _tcscpy(szFileName, szFile);

        // If an NT native name, clear it
        if(!_tcsncmp(szFileName, _T("\\??\\"), 4))
            memmove(szFileName, &szFileName[4], (sizeof(szFileName) - 4 * sizeof(TCHAR)));
    }

    // Load the filters. They must be separated by "|"
    LoadString(g_hInst, nIDFilters, szFilters, _tsize(szFilters));
    for(int i = 0; szFilters[i] != 0; i++)
    {
        if(szFilters[i] == _T('|'))
            szFilters[i] = 0;
    }
    
    ZeroMemory(&ofn, sizeof(OPENFILENAME));
    ofn.lStructSize     = sizeof(OPENFILENAME);
    ofn.hwndOwner       = hParent;
    ofn.hInstance       = g_hInst;
    ofn.lpstrInitialDir = szInitialDir;
    ofn.lpstrFilter     = szFilters;
    ofn.lpstrFile       = szFileName;
    ofn.nMaxFile        = _tsize(szFileName);
    ofn.lpstrTitle      = szTitle;
    ofn.Flags           = OFN_ENABLESIZING | OFN_EXPLORER | OFN_FILEMUSTEXIST | OFN_HIDEREADONLY | OFN_NOCHANGEDIR | OFN_NONETWORKBUTTON | OFN_PATHMUSTEXIST;
    bResult = GetOpenFileName(&ofn);

    if(bResult)
    {
        szPlainName = szFileName;

        // Store the initial dir
        if(szDir != NULL)
        {
            _tcscpy(szInitialDir, szFileName);
            szPlainName = GetPlainName(szInitialDir);
            *(szPlainName - 1) = 0;

            if((DWORD_PTR)szDir <= 0x10000)
                SetDlgItemText(hParent, (UINT)(UINT_PTR)szDir, szInitialDir);
            else
                _tcscpy(szDir, szInitialDir);
        }

        // Store the file name
        if(szFile != NULL)
        {
            if((DWORD_PTR)szFile <= 0x10000)
                SetDlgItemText(hParent, (UINT)(UINT_PTR)szFile, szPlainName);
            else
                _tcscpy(szFile, szPlainName);
        }
    }

    return bResult;
}
