/*****************************************************************************/
/* CheckBoxMask.cpp                       Copyright (c) Ladislav Zezula 2004 */
/*---------------------------------------------------------------------------*/
/* Getting & setting mask value frm an array of checkboxes                   */
/*---------------------------------------------------------------------------*/
/*   Date    Ver   Who  Comment                                              */
/* --------  ----  ---  -------                                              */
/* 29.05.04  1.00  Lad  The first version of CheckBoxMask.cpp                */
/*****************************************************************************/

#define __UTILS_SELF
#include "Utils.h"
#include <windowsx.h>

static const TCHAR * szButtonClass = _T("Button");

// Gets a mask value from check boxes. The checkboxes must follow each other
// on the dialog layout, be sure that they do !!!
DWORD GetCheckBoxMask(HWND hDlg, UINT nIDFirst)
{
    TCHAR szClassName[32];
    HWND  hCheckBox = GetDlgItem(hDlg, nIDFirst);
    DWORD dwStyles = GetWindowLong(hCheckBox, GWL_STYLE);
    DWORD dwMask = 0;              // Checkboxes mask
    DWORD dwBit = 1;

    // Now pass all the controls which are groupbox and have NOT the "Group" flag
    while(hCheckBox != NULL)
    {
        // Ensure that it is really a checkbox
        GetClassName(hCheckBox, szClassName, _tsize(szClassName));
        if(!_tcsicmp(szClassName, szButtonClass) && (dwStyles & 0x0F) == BS_AUTOCHECKBOX)
        {
            // If the radio button is checked, return its index.
            if(Button_GetCheck(hCheckBox) == BST_CHECKED)
                dwMask |= dwBit;
        }

        // Move to the next control.
        // If the next has the group flag set, break the loop
        hCheckBox = GetWindow(hCheckBox, GW_HWNDNEXT);
        dwStyles = GetWindowLong(hCheckBox, GWL_STYLE);
        dwBit <<= 1;
        if(dwStyles & WS_GROUP)
            break;
    }

    // Return the mask
    return dwMask;
}

// Sets a mask for check boxes.
// The checkboxes must follow each other, be sure that they do !!!
void SetCheckBoxMask(HWND hDlg, UINT nIDFirst, DWORD dwMask, DWORD dwEnabled)
{
    TCHAR szClassName[32];
    HWND  hCheckBox = GetDlgItem(hDlg, nIDFirst);
    DWORD dwStyles = GetWindowLong(hCheckBox, GWL_STYLE);
    DWORD dwBit = 1;              // Checkbox bit #

    // Now pass all the controls which are groupbox and have NOT the "Group" flag
    while(hCheckBox != NULL)
    {
        // Ensure that it is really a checkbox button
        GetClassName(hCheckBox, szClassName, _tsize(szClassName));
        if(!_tcsicmp(szClassName, szButtonClass) && (dwStyles & 0x0F) == BS_AUTOCHECKBOX)
        {
            Button_SetCheck(hCheckBox, (dwBit & dwMask) ? BST_CHECKED : BST_UNCHECKED);
            EnableWindow(hCheckBox, (dwBit & dwEnabled) ? TRUE : FALSE);
        }

        // Move to the next control.
        // If the next has the group flag set, break the loop
        hCheckBox = GetWindow(hCheckBox, GW_HWNDNEXT);
        dwStyles = GetWindowLong(hCheckBox, GWL_STYLE);
        dwBit <<= 1;
        if(dwStyles & WS_GROUP)
            break;
    }
}
