/*****************************************************************************/
/* CompareStringWildcard.cpp              Copyright (c) Ladislav Zezula 2004 */
/*---------------------------------------------------------------------------*/
/* Description:                                                              */
/*---------------------------------------------------------------------------*/
/*   Date    Ver   Who  Comment                                              */
/* --------  ----  ---  -------                                              */
/* 30.12.04  1.00  Lad  The first version of CompareStringWildcard.cpp       */
/*****************************************************************************/

#define __UTILS_SELF
#include "Utils.h"

//
// This function compares a string with a wildcard search string.
// returns TRUE, when the string matches with the wildcard.
//

BOOL CompareStringWildCard(LPCTSTR szString, LPCTSTR szWildCard)
{
    LPCTSTR szTemp;                     // Temporary helper pointer
    int nResult = 0;                    // For memcmp return values
    int nMustNotMatch = 0;              // Number of following chars int szString,
                                        // which must not match with szWildCard
    int nMustMatch = 0;                 // Number of the following characters,
                                        // which must match

    // When the string is empty, it does match "*" only
    if(*szString == 0)
    {
        return (szWildCard[0] == _T('*') && szWildCard[1] == 0);
    }

    // When the mask is empty, it matches to every wildcard
    if(szWildCard == NULL || *szWildCard == 0)
        return FALSE;

    // Do normal test
    for(;;)
    {
        switch(*szWildCard)
        {
            case _T('*'): // Means "every number of characters"
                // Skip all asterisks
                while(*szWildCard == _T('*'))
                    szWildCard++;

                // When no more characters in wildcard, it means that the strings match
                if(*szWildCard == 0)
                    return TRUE;

                // The next N characters must not agree
                nMustNotMatch |= 0x70000000;
                break;
            
            case _T('?'):  // Means "One or no character"
                while(*szWildCard == _T('?'))
                {
                    nMustNotMatch++;
                    szWildCard++;
                }
                break;

            default:
                // If the two characters match
//              if(*szString == *szWildCard)
                if(CompareString(LOCALE_USER_DEFAULT, NORM_IGNORECASE, szString, 1, szWildCard, 1) == CSTR_EQUAL)
                {
                    // When end of string, they agree
                    if(*szString == 0)
                        return TRUE;

                    nMustNotMatch = 0;
                    szWildCard++;
                    szString++;
                    break;
                }

                // If the next character must match, the string does not match
                if(nMustNotMatch == 0)
                    return FALSE;

                // Count the characters which must match after characters
                // that must not match
                szTemp = szWildCard;
                nMustMatch = 0;
                while(*szTemp != 0 && *szTemp != _T('*') && *szTemp != _T('?'))
                {
                    nMustMatch++;
                    szTemp++;
                }

                // Now skip characters from szString up to number of chars
                // that must not match
                nResult = -1;
                while(nMustNotMatch > 0 && *szString != 0)
                {
                    if((nResult = _tcsnicmp(szString, szWildCard, nMustMatch)) == 0)
                        break;
                    
                    szString++;
                    nMustNotMatch--;
                }

                // Make one more comparison
                if(nMustNotMatch == 0)
                    nResult = _tcsnicmp(szString, szWildCard, nMustMatch);

                // If a match has been found, continue the search
                if(nResult == 0)
                {
                    nMustNotMatch = 0;
                    szWildCard += nMustMatch;
                    szString   += nMustMatch;
                    break;
                }
                return FALSE;
        }
    }
}
