/*****************************************************************************/
/* FindProcess.cpp                        Copyright (c) Ladislav Zezula 2004 */
/*---------------------------------------------------------------------------*/
/* A function that finds a process by its name.                              */
/*---------------------------------------------------------------------------*/
/*   Date    Ver   Who  Comment                                              */
/* --------  ----  ---  -------                                              */
/* 31.05.04  1.00  Lad  The first version of FindProcess.cpp                 */
/*****************************************************************************/

#define __UTILS_SELF
#include "Utils.h"
#include <Tlhelp32.h>

//-----------------------------------------------------------------------------
// Defines

#define INVALID_PROCESS_ID (DWORD)-1;

//-----------------------------------------------------------------------------
// These functions walk all the processes test if the specified one is not there.
// We have to retrieve the Module32First and Module32Next dynamically, because of
// WinNT 4.0. This function always returns a name, even if the module cannot be found.

typedef HANDLE(WINAPI * CREATESNAP)(DWORD, DWORD);
typedef BOOL  (WINAPI * PROCESSWALK)(HANDLE, PROCESSENTRY32 *);
typedef BOOL  (WINAPI * CLOSESNAP)(HANDLE);

// Types for the PSAPI.dll
typedef BOOL  (WINAPI * ENUMPROCESSES)(DWORD *, DWORD, DWORD *);
typedef BOOL  (WINAPI * ENUMMODULES)(HANDLE, HMODULE *, DWORD, LPDWORD);
typedef DWORD (WINAPI * GETMODFNAMEEXW)(HANDLE, HMODULE, LPTSTR, DWORD);

// Walks through processes using ToolHelp snapshots
static DWORD WalkThoughProcesses1(HINSTANCE hKernel32, const TCHAR * szAppName)
{
    PROCESSENTRY32 pe;
    CREATESNAP  CreateSnapshot = (CREATESNAP)GetProcAddress(hKernel32, "CreateToolhelp32Snapshot");
    PROCESSWALK Process32First = (PROCESSWALK)GetProcAddress(hKernel32, "Process32FirstW");
    PROCESSWALK Process32Next  = (PROCESSWALK)GetProcAddress(hKernel32, "Process32NextW");
    HANDLE hSnap1;                  // Process snapshot
    DWORD dwProcessID = INVALID_PROCESS_ID;
    BOOL bFoundPs;

    szAppName = _tcsrchr(szAppName, _T('\\'));
    if(szAppName == NULL)
        return FALSE;
    szAppName++;

    // Create the process snapshot and walk through the processes
    pe.dwSize = sizeof(PROCESSENTRY32);
    hSnap1 = CreateSnapshot(TH32CS_SNAPPROCESS, 0);

    for(bFoundPs = Process32First(hSnap1, &pe);
        bFoundPs == TRUE;
        bFoundPs = Process32Next(hSnap1, &pe))
    {
        if(!_tcsicmp(szAppName, pe.szExeFile))
        {
            dwProcessID = pe.th32ProcessID;
            break;
        }
    }
    CloseHandle(hSnap1);
    return dwProcessID;
}

// Walks through processes using psapi.dll
static DWORD WalkThoughProcesses2(HINSTANCE hPSAPI, const TCHAR * szAppName)
{
    // Version for WinNT 4.0
    GETMODFNAMEEXW GetModuleFileNameEx = (GETMODFNAMEEXW)GetProcAddress(hPSAPI, "GetModuleFileNameExW");
    ENUMPROCESSES EnumProcesses = (ENUMPROCESSES)GetProcAddress(hPSAPI, "EnumProcesses");
    ENUMMODULES EnumProcessModules = (ENUMMODULES)GetProcAddress(hPSAPI, "EnumProcessModules");
    HANDLE      hProcess;
    TCHAR       szFileName[MAX_PATH] = _T("");
    DWORD       dwProcessIDs[256];
    HINSTANCE   hModules[256];
    DWORD       dwProcessID = INVALID_PROCESS_ID;
    DWORD       dwProcesses;
    DWORD       dwModules;
    BOOL        bResult = FALSE;

    szAppName = _tcsrchr(szAppName, _T('\\'));
    if(szAppName == NULL)
        return FALSE;

    // Retrieve the number of modules
    EnumProcesses(dwProcessIDs, sizeof(dwProcessIDs), &dwProcesses);
    dwProcesses /= sizeof(DWORD);

    // Walk all returned module handles
    for(DWORD i = 0; i < dwProcesses && bResult == FALSE; i++)
    {
        // Open the process (we need handle for the next work)
        if((hProcess = OpenProcess(PROCESS_QUERY_INFORMATION | PROCESS_VM_READ, FALSE, dwProcessIDs[i])) == NULL)
            continue;

        // Get the first module of the process
        dwModules = 0;
        EnumProcessModules(hProcess, hModules, sizeof(hModules), &dwModules);
        dwModules /= sizeof(HINSTANCE);

        for(DWORD j = 0; j < dwModules; j++)
        {
            TCHAR * szTemp = szFileName;

            GetModuleFileNameEx(hProcess, hModules[j], szFileName, sizeof(szFileName) / sizeof(TCHAR));
            szTemp = _tcsrchr(szTemp, _T('\\'));
            if(!_tcsicmp(szAppName, szTemp))
            {
                dwProcessID = dwProcessIDs[i];
                bResult = TRUE;
                break;
            }
        }
        CloseHandle(hProcess);
    }
    return dwProcessID;
}

//-----------------------------------------------------------------------------
// Public functions

//
// Searches all available processes by given name. If the name is found, 
// the function returns the process ID.
//

DWORD FindProcess(const TCHAR * szAppName)
{
    HINSTANCE hKernel32 = LoadLibrary(_T("Kernel32.dll"));
    HINSTANCE hPSAPI = LoadLibrary(_T("PSAPI.dll"));
    DWORD dwProcessID = INVALID_PROCESS_ID;

    // Test for Win9x, Win2000, WinXP
    if(GetProcAddress(hKernel32, "CreateToolhelp32Snapshot") != NULL)
        dwProcessID = WalkThoughProcesses1(hKernel32, szAppName);
    
    // Test for WinNT 4.0
    else if(GetProcAddress(hPSAPI, "EnumProcesses") != NULL)
        dwProcessID = WalkThoughProcesses2(hPSAPI, szAppName);

    // Cleanup and exit.
    FreeLibrary(hPSAPI);
    FreeLibrary(hKernel32);
    return dwProcessID;
}

