/*****************************************************************************/
/* ForcePathExist.cpp                     Copyright (c) Ladislav Zezula 2004 */
/*---------------------------------------------------------------------------*/
/* Description:                                                              */
/*---------------------------------------------------------------------------*/
/*   Date    Ver   Who  Comment                                              */
/* --------  ----  ---  -------                                              */
/* 30.05.04  1.00  Lad  The first version of ForcePathExist.cpp              */
/*****************************************************************************/

#define __UTILS_SELF
#include "Utils.h"

// Ensures that the path for the extracted file already exists
// or creates it.
int ForcePathExist(const TCHAR * szFileName, BOOL bIsDirectory)
{
    TCHAR szFullPath[MAX_PATH] = _T("");
    TCHAR * szTemp;
    int  nError = ERROR_SUCCESS;

    // Network paths - remove the server part
    if(szFileName[0] == _T('\\') && szFileName[1] == _T('\\'))
    {
        szTemp = (TCHAR *)_tcschr(&szFileName[2], _T('\\'));
        if(szTemp == NULL)
            return ERROR_INVALID_PARAMETER;

        _tcsncpy(szFullPath, szFileName, (szTemp - szFileName));
        szFullPath[szTemp - szFileName] = 0;
        szFileName = szTemp + 1;
    }

    // Pre-init the path with search path
    while((szTemp = (TCHAR *)_tcschr(szFileName, _T('\\'))) != NULL)
    {
        BOOL bRootDir = FALSE;

        // Append the (next) path part
        AddBackslash(szFullPath);
        _tcsncat(szFullPath, szFileName, (szTemp - szFileName));

        if(szFullPath[1] == _T(':') && szFullPath[2] == 0)
            bRootDir = TRUE;
        if(szFullPath[0] == _T('\\') && szFullPath[1] == _T('\\') && szFullPath[2] == 0)
            bRootDir = TRUE;

        // Create the directory, except for root dir
        if(bRootDir == FALSE)
        {
            if(!CreateDirectory(szFullPath, NULL))
            {
                nError = GetLastError();
                if(nError != ERROR_ALREADY_EXISTS)
                    break;
                nError = ERROR_SUCCESS;
            }
        }
        szFileName = szTemp + 1;
    }

    // If it is a directory, force create the last part
    if(nError == ERROR_SUCCESS && bIsDirectory && *szFileName != 0)
    {
        AddBackslash(szFullPath);
        _tcscat(szFullPath, szFileName);

        if(!CreateDirectory(szFullPath, NULL))
        {
            nError = GetLastError();
            if(nError == ERROR_ALREADY_EXISTS)
                nError = ERROR_SUCCESS;
        }
    }
    return nError;
}



