/*****************************************************************************/
/* HandleButtons.cpp                      Copyright (c) Ladislav Zezula 2004 */
/*---------------------------------------------------------------------------*/
/* Description:                                                              */
/*---------------------------------------------------------------------------*/
/*   Date    Ver   Who  Comment                                              */
/* --------  ----  ---  -------                                              */
/* 30.05.04  1.00  Lad  The first version of HandleButtons.cpp               */
/*****************************************************************************/

#define __UTILS_SELF
#include "Utils.h"

//-----------------------------------------------------------------------------
// Handles the 'OK', 'Cancel' and 'Apply' buttons.
// 
//   1. If Apply was pressed and changed is TRUE, calls the SaveDialog function for store dialog content
//   2. If OK was pressed and changed is TRUE, calls SaveDialog and ends the dialog with IDOK
//   3. If OK was pressed and changed is FALSE, calls ends the dialog with IDCANCEL.
//   4. If Cancel was pressed, ends the dialog with IDCANCEL.
//
//  The 'SaveDialog' must store all dialog setting in memory pointed by the given parameter.
//  and return TRUE if storing was OK or FALSE if some error (like invalid number value etc.)
//
//  Returns TRUE is the dialog will be closed, FALSE if not.

BOOL HandleButtons(HWND hDlg, WPARAM wParam, void * SaveProc, BOOL bChanged)
{
#ifndef IDAPPLY
#define IDAPPLY   60000                 // Prevent before syntax errors id no IDAPPLY used
#endif

    BOOL (*SaveDialog)(HWND) = (BOOL(*)(HWND))SaveProc;
    int nNotify = HIWORD(wParam);       // Notify code
    int nIDCtrl = LOWORD(wParam);       // Control ID

    // Do something only if button was pressed
    if(nNotify == BN_CLICKED)
    {
        switch(nIDCtrl)
        {
            // If 'Apply' was pressed and 'changed' is TRUE, call the SaveDialog function    
            case IDAPPLY:
                if(SaveDialog != NULL && bChanged == TRUE)
                    SaveDialog(hDlg);
                return FALSE;

            case IDOK:
                // If storing dialog failed, do nothing.
                if(SaveDialog != NULL && bChanged == TRUE && SaveDialog(hDlg) == FALSE)
                    return FALSE;

                // If not changed, set IDCANCEL as result code
                if(bChanged == FALSE)
                    nIDCtrl = IDCANCEL;
                // Here is not the 'break' statement

            // If 'Cancel' pressed, simply end the dialog.
            case IDCANCEL:
            case IDCLOSE:
                EndDialog(hDlg, nIDCtrl);
                return TRUE;
        }
    }
    // Nothing to do.
    return FALSE;
}
