/*****************************************************************************/
/* InitDialogControls.cpp                 Copyright (c) Ladislav Zezula 2004 */
/*---------------------------------------------------------------------------*/
/* Function initializing the dialog box controls (listboxes and comboboxes)  */
/*---------------------------------------------------------------------------*/
/*   Date    Ver   Who  Comment                                              */
/* --------  ----  ---  -------                                              */
/* 29.05.04  1.00  Lad  The first version of InitDialogControls.cpp          */
/*****************************************************************************/

#define __UTILS_SELF
#include "Utils.h"

//-----------------------------------------------------------------------------
// Defines

#define RT_DLGINIT         MAKEINTRESOURCE(240)
#define WIN16_LB_ADDSTRING 0x0401
#define WIN16_CB_ADDSTRING 0x0403

//-----------------------------------------------------------------------------
// Structures

typedef struct
{
    WORD  ctrl;                          // Control ID
    WORD  msg;                           // Message ID
    DWORD dataSize;                      // Size of the following data
    // Followed by zero-terminated data
} TDlgInitData;

//-----------------------------------------------------------------------------
// Functions

// Loads listbox or combobox items from resources
int InitDialogControls(HWND hDlg, LPCTSTR lpszResourceName)
{
    HINSTANCE hInst  = (HINSTANCE)GetWindowLongPtr(hDlg, GWLP_HINSTANCE);
    HANDLE hResource = NULL;
    HRSRC  hDlgInit  = FindResource(hInst, lpszResourceName, RT_DLGINIT);

    // Try to find dialog initialization data
    if(hDlgInit != NULL)
    {
		// Load the data
        if((hResource = LoadResource(hInst, hDlgInit)) != NULL)
        {
            // Lock the data and obtain 
            TDlgInitData * init = (TDlgInitData *)LockResource(hResource);

            while(init->ctrl != 0)
            {
                char * data = (char *)init + sizeof(TDlgInitData);
                UINT   msg  = init->msg;

                // Convert old Win16 messages
                if(msg == WIN16_LB_ADDSTRING)
                    msg = LB_ADDSTRING;
                if(msg == WIN16_CB_ADDSTRING)
                    msg = CB_ADDSTRING;

                if(msg == LB_ADDSTRING || msg == CB_ADDSTRING)
                    SendDlgItemMessageA(hDlg, init->ctrl, msg, 0, (LPARAM)data);

                // Move to the next data item
                init = (TDlgInitData *)(data + init->dataSize);
            }
            UnlockResource(hResource);
        }
        FreeResource(hResource);
    }
    return 0;
}
