/*****************************************************************************/
/* MessageBoxRc.cpp                       Copyright (c) Ladislav Zezula 2004 */
/*---------------------------------------------------------------------------*/
/* Description:                                                              */
/*---------------------------------------------------------------------------*/
/*   Date    Ver   Who  Comment                                              */
/* --------  ----  ---  -------                                              */
/* 29.05.04  1.00  Lad  The first version of MessageBoxRc.cpp                */
/*****************************************************************************/

#define __UTILS_SELF
#include "Utils.h"

//-----------------------------------------------------------------------------
// Shows a message box using resource strings
//
// Parameters
// 
//   hParent - handle to the parent of the message box
//
//   nIDCaption - identifier or pointer to the string that will be shown in the
//       title. It can be either well-known string ID (see Strings.cpp from Utils
//       library), resource string ID or pointer to a string cast to UINT.
//
//   nIDText - identifier or pointer to the string that will be shown in the
//       message text. It can be either resource string ID or pointer
//       to a string cast to UINT.
//

int MessageBoxRc(HWND hParent, UINT_PTR nIDCaption, UINT_PTR nIDText, ...)
{
    LPCTSTR szCaption;
    va_list argList;
    TCHAR szCaptionBuffer[256] = _T("");
    TCHAR szTextBuffer[1024] = _T("");
    TCHAR szText[1024] = _T("");
    UINT  nButtons = MB_OK | MB_ICONINFORMATION;

    // Get dialog caption
    szCaption = (LPCTSTR)nIDCaption;
    if(nIDCaption <= 0x00010000)
    {
        // Try to load the string from standard string set.
        // If not given, try to load it from the resources.
        if((szCaption = GetString(nIDCaption)) == NULL)
        {
            if(LoadString(g_hInst, (UINT)nIDCaption, szCaptionBuffer, _tsize(szCaptionBuffer)) != 0)
                szCaption = szCaptionBuffer;
            else
                szCaption = _T("* CAPTION NOT LOADED *");
        }
    }

    // Get message text
    if(nIDText <= 0x00010000)
    {
        if(LoadString(g_hInst, (UINT)nIDText, szTextBuffer, _tsize(szTextBuffer)) == 0)
            _tcscpy(szTextBuffer, _T("* MESSAGE TEXT NOT LOADED *"));
    }
    else
    {
        _tcsncpy(szTextBuffer, (LPCTSTR)nIDText, _tsize(szTextBuffer));
        szTextBuffer[_tsize(szTextBuffer) - 1] = 0;
    }

    // Format the message text
    va_start(argList, nIDText);
    _vstprintf(szText, szTextBuffer, argList);
    va_end(argList);

    // Get buttons
    switch(nIDCaption)
    {
        case IDS_INFO:       nButtons = MB_OK | MB_ICONINFORMATION; break;
        case IDS_ERROR:      nButtons = MB_OK | MB_ICONERROR; break;
        case IDS_QUESTION:   nButtons = MB_YESNO | MB_ICONQUESTION; break;
        case IDS_CONFIRM:    nButtons = MB_YESNOCANCEL | MB_ICONQUESTION; break;
        case IDS_WARNING:    nButtons = MB_OK | MB_ICONWARNING; break;
        case IDS_REQUEST:    nButtons = MB_OKCANCEL | MB_ICONWARNING; break;
        case IDS_RETRYCANCEL:nButtons = MB_RETRYCANCEL | MB_ICONWARNING; break;
        default :            nButtons = MB_OK | MB_ICONASTERISK; break;
    }

    return MessageBox(hParent, szText, szCaption, nButtons);
}
