/*****************************************************************************/
/* ShowSystrayBaloon.cpp                  Copyright (c) Ladislav Zezula 2004 */
/*---------------------------------------------------------------------------*/
/* Description:                                                              */
/*---------------------------------------------------------------------------*/
/*   Date    Ver   Who  Comment                                              */
/* --------  ----  ---  -------                                              */
/* 02.06.04  1.00  Lad  The first version of ShowSystrayBaloon.cpp           */
/*****************************************************************************/

#define __UTILS_SELF
#include "Utils.h"

//-----------------------------------------------------------------------------
// Defines

#ifndef NIF_INFO
#define NIF_INFO        0x00000010
#endif

#ifndef NIIF_INFO
#define NIIF_INFO     0x00000001
#endif

#ifndef NIIF_WARNING
#define NIIF_WARNING  0x00000002
#endif

#ifndef NIFF_ERROR
#define NIIF_ERROR    0x00000003
#endif

//-----------------------------------------------------------------------------
// Structures

struct TNotifyIconData
{
    DWORD cbSize;
    HWND  hWnd;
    UINT  uID;
    UINT  uFlags;
    UINT  uCallbackMessage;
    HICON hIcon;
};

struct TNotifyIconData4 : public TNotifyIconData
{
    TCHAR szTip[64];
};

struct TNotifyIconData5 : public TNotifyIconData
{
    TCHAR szTip[128];
    DWORD dwState;
    DWORD dwStateMask;
    TCHAR szInfo[256];
    union
    {
        UINT  uTimeout;
        UINT  uVersion;
    };
    TCHAR szInfoTitle[64];
    DWORD dwInfoFlags;
};

struct TNotifyIconData6 : public TNotifyIconData5
{
    GUID guidItem;
};

//-----------------------------------------------------------------------------
// Local variables

static const TCHAR * szMsgBoxTitle = NULL;
static DWORD dwShell32Version = 0;

//-----------------------------------------------------------------------------
// Local functions

static VOID CALLBACK TimerProc(HWND hWnd, UINT, UINT_PTR idEvent, DWORD)
{
    HWND hMsgBox = FindWindow(NULL, szMsgBoxTitle);

    if(hMsgBox != NULL)
        DestroyWindow(hMsgBox);
    KillTimer(hWnd, idEvent);
    szMsgBoxTitle = NULL;
}

//-----------------------------------------------------------------------------
// Public functions

int ShowSystrayBaloon(HWND hDlg, UINT nIDIcon, UINT nIDTitle, UINT nIDText, ...)
{
    TCHAR * szTitle = (TCHAR *)GetString(nIDTitle);
    TCHAR szTitleBuff[256] = _T("");
    TCHAR szText[512] = _T("");
    TCHAR szFmt[256] = _T("");
    UINT nTimeout = 2000;
    va_list argList;

    // Verify the version of Shell32.dll. This is because
    // only Shell32.dll v 5+ support baloon tooltips.
    if(dwShell32Version == 0)
    {
        LARGE_INTEGER Version;

        GetModuleVersion(_T("Shell32.dll"), &Version);
        dwShell32Version = Version.HighPart;
    }

    // If the string could not be retrieved fro our internal strings,
    // try load it from the resources
    if(szTitle == NULL)
    {
        LoadString(g_hInst, nIDTitle, szTitleBuff, _tsize(szTitleBuff));
        szTitle  = szTitleBuff;
        nIDTitle = IDS_INFO;
    }

    // Set the baloon text
    va_start(argList, nIDText);
    LoadString(g_hInst, nIDText, szFmt, _tsize(szFmt));
    _vstprintf(szText, szFmt, argList);
    va_end(argList);

    // If the shell version if greater or equal to 5,
    // show the baloon tooltip.
    if(dwShell32Version >= 0x00050000)
    {
        TNotifyIconData5 nid;

        ZeroMemory(&nid, sizeof(TNotifyIconData5));
        nid.cbSize      = sizeof(TNotifyIconData5);
        nid.hWnd        = hDlg;
        nid.uID         = nIDIcon;
        nid.uFlags      = NIF_INFO;
        nid.uTimeout    = nTimeout;

        // Set the right type
        switch(nIDTitle)
        {
            case IDS_INFO:
                nid.dwInfoFlags = NIIF_INFO;
                break;

            case IDS_WARNING:
                nid.dwInfoFlags = NIIF_WARNING;
                break;

            case IDS_ERROR:
                nid.dwInfoFlags = NIIF_ERROR;
                break;
        }

        // Set the title and text
        _tcscpy(nid.szInfoTitle, szTitle);
        _tcsncpy(nid.szInfo, szText, _tsize(nid.szInfo));
        
        // Show the baloon
        Shell_NotifyIcon(NIM_MODIFY, (NOTIFYICONDATA *)&nid);
        return TRUE;
    }

    // On shells 4.x, show timed message box
    szMsgBoxTitle = szTitle;
    SetTimer(hDlg, 0, nTimeout, TimerProc);
    MessageBox(NULL, szText, szTitle, MB_OK | MB_ICONASTERISK);
    return TRUE;
}
