/*****************************************************************************/
/* TToolTip.cpp                           Copyright (c) Ladislav Zezula 2004 */
/*---------------------------------------------------------------------------*/
/* Description:                                                              */
/*---------------------------------------------------------------------------*/
/*   Date    Ver   Who  Comment                                              */
/* --------  ----  ---  -------                                              */
/* 07.06.04  1.00  Lad  The first version of TToolTip.cpp                    */
/*****************************************************************************/

#define __UTILS_SELF
#include "Utils.h"
#include "TToolTip.h"

//-----------------------------------------------------------------------------
// Constructor and destructor

TToolTip::TToolTip(HWND hWnd)
{
    int nMaxWidth = GetSystemMetrics(SM_CXSCREEN);

    m_nIdCounter = 1;
    m_hParent = hWnd;
    
    // Create the tooltip window
    m_hToolTip = CreateWindowEx(WS_EX_TOPMOST,
                                TOOLTIPS_CLASS,
                                NULL,
                                WS_POPUP | TTS_NOPREFIX | TTS_ALWAYSTIP,		
                                CW_USEDEFAULT, CW_USEDEFAULT, CW_USEDEFAULT, CW_USEDEFAULT,
                                hWnd,
                                NULL,
                                g_hInst,
                                NULL);
    
    SetWindowPos(m_hToolTip, HWND_TOPMOST, 0, 0, 0, 0, SWP_NOMOVE | SWP_NOSIZE | SWP_NOACTIVATE);
    
    // Multiline support
    SendMessage(m_hToolTip, TTM_SETMAXTIPWIDTH, 0, nMaxWidth);

    // Set hide timeout to one minute
    SendMessage(m_hToolTip, TTM_SETDELAYTIME, TTDT_AUTOPOP, MAKELONG(60000, 0));
}

TToolTip::~TToolTip()
{
    if(m_hToolTip != NULL)
        DestroyWindow(m_hToolTip);
}

//-----------------------------------------------------------------------------
// Adding tooltips for child windows and/or rectangles

LRESULT TToolTip::AddTipForChild(HWND hWnd, UINT_PTR nIDString)
{
    TOOLINFO ti;
    TCHAR szClassName[128] = _T("");
    POINT ptClient = {0, 0};
    TCHAR szText[256] = _T("");
    BOOL bIsStatic = FALSE;

    assert(GetParent(hWnd) == m_hParent);
    GetClassName(hWnd, szClassName, _tsize(szClassName));
    if(!_tcsicmp(szClassName, _T("Static")))
        bIsStatic = TRUE;

    // Get the text of the tooltip
    if(nIDString < 0x10000)
        LoadString(g_hInst, (UINT)nIDString, szText, _tsize(szText));
    else
        _tcscpy(szText, (TCHAR *)nIDString);

    // Fill the TOOLINFO structure to be 
    ZeroMemory(&ti, sizeof(TOOLINFO));
    ti.cbSize   = sizeof(TOOLINFO);
    ti.uFlags   = TTF_SUBCLASS;
    ti.hinst    = g_hInst;
    ti.lpszText = szText;
    ti.hwnd     = m_hParent;
    ti.uId      = (bIsStatic == FALSE) ? (UINT_PTR)hWnd : m_nIdCounter++;
    if(bIsStatic == FALSE)
        ti.uFlags |= TTF_IDISHWND;

    ClientToScreen(m_hParent, &ptClient);
    GetWindowRect(hWnd, &ti.rect);
    ti.rect.left   -= ptClient.x;
    ti.rect.top    -= ptClient.y;
    ti.rect.right  -= ptClient.x;
    ti.rect.bottom -= ptClient.y;
    return SendMessage(m_hToolTip, TTM_ADDTOOL, 0, (LPARAM)&ti);	
}

LRESULT TToolTip::AddTipForChild(HWND hDlg, UINT nIDCtrl, UINT nIDString)
{
    return AddTipForChild(GetDlgItem(hDlg, nIDCtrl), nIDString);
}

LRESULT TToolTip::AddTipForRect(RECT & rect, UINT_PTR nIDString)
{
    TOOLINFO ti;
    TCHAR szText[256] = _T("");

    // Get the text of the tooltip
    if(nIDString < 0x10000)
        LoadString(g_hInst, (UINT)nIDString, szText, _tsize(szText));
    else
        _tcscpy(szText, (TCHAR *)nIDString);

    // Fill the TOOLINFO structure to be 
    ZeroMemory(&ti, sizeof(TOOLINFO));
    ti.cbSize   = sizeof(TOOLINFO);
    ti.uFlags   = TTF_SUBCLASS;
    ti.hinst    = g_hInst;
    ti.lpszText = szText;
    ti.hwnd     = m_hParent;
    ti.uId      = m_nIdCounter++;
    ti.rect     = rect;
    return SendMessage(m_hToolTip, TTM_ADDTOOL, 0, (LPARAM)&ti);	
}

//void TToolTip::RemoveAll()
//{}
